@@ -20,6 +20,8 @@
 #include <ctype.h>
 #include <time.h>
 
+#include <haproxy/compiler.h>
+
 #include <import/eb32tree.h>
 #include <import/eb64tree.h>
 #include <import/ebistree.h>
@@ -147,13 +149,14 @@ void usage(FILE *output, const char *msg)
 {
 	fprintf(output,
 		"%s"
-		"Usage: halog [-h|--help] for long help\n"
-		"       halog [-q] [-c] [-m <lines>]\n"
-		"       {-cc|-gt|-pct|-st|-tc|-srv|-u|-uc|-ue|-ua|-ut|-uao|-uto|-uba|-ubt|-ic\n"
-		"           |-hdr <block>:<field>\n"
-		"       }\n"
-		"       [-s <skip>] [-e|-E] [-H] [-rt|-RT <time>] [-ad <delay>] [-ac <count>] [-query]\n"
-		"       [-v] [-Q|-QS] [-tcn|-TCN <termcode>] [ -hs|-HS [min][:[max]] ] [ -time [min][:[max]] <log\n"
+		"Usage:\n"
+		"  halog [-h|--help] for long help\n"
+		"  halog [input_filters]* [modifiers]* [output_format] < log\n"
+		"    inp = [-e|-E] [-H] [-Q|-QS] [-rt|-RT <time>] [-ad <delay>] [-ac <count>]\n"
+		"          [-hs|-HS [min][:[max]]] [-tcn|-TCN <termcode>] [-time [min][:[max]]]\n"
+		"    mod = [-q] [-v] [-m <lines>] [-s <skipflds>] [-query]\n"
+		"    out = {-c|-u|-uc|-ue|-ua|-ut|-uao|-uto|-uba|-ubt|-hdr <block>:<field>|\n"
+		"           -cc|-gt|-pct|-st|-tc|-srv|-ic}\n"
 		"\n",
 		msg ? msg : ""
 		);
@@ -169,7 +172,7 @@ void help()
 {
 	usage(stdout, NULL);
 	printf(
-	       "Input filters (several filters may be combined) :\n"
+	       "Input filters - several filters may be combined\n"
 	       " -H                      only match lines containing HTTP logs (ignore TCP)\n"
 	       " -E                      only match lines without any error (no 5xx status)\n"
 	       " -e                      only match lines with errors (status 5xx or negative)\n"
@@ -189,7 +192,7 @@ void help()
 	       "                         you can also use -n to start from earlier then field %d\n"
 	       " -query                  preserve the query string for per-URL (-u*) statistics\n"
 	       "\n"
-	       "Output filters - only one may be used at a time\n"
+	       "Output format - only one may be used at a time\n"
 	       " -c    only report the number of lines that would have been printed\n"
 	       " -pct  output connect and response times percentiles\n"
 	       " -st   output number of requests per HTTP status code\n"
@@ -204,7 +207,7 @@ void help()
 	       "       -uao, -uto: average times computed on valid ('OK') requests\n"
 	       "       -uba, -ubt: average bytes returned, total bytes returned\n"
 	       " -hdr  output captured header at the given <block>:<field>\n",
-	       SOURCE_FIELD,SOURCE_FIELD
+	       (int)SOURCE_FIELD, (int)SOURCE_FIELD
 	       );
 	exit(0);
 }
@@ -252,6 +255,16 @@ const char *field_stop(const char *p)
 }
 #endif
 
+/* return non-zero if the argument contains at least one zero byte. See principle above. */
+static inline __attribute__((unused)) unsigned long long has_zero64(unsigned long long x)
+{
+	unsigned long long y;
+
+	y = x - 0x0101010101010101ULL; /* generate a carry */
+	y &= ~x;                       /* clear the bits that were already set */
+	return y & 0x8080808080808080ULL;
+}
+
 /* return field <field> (starting from 1) in string <p>. Only consider
  * contiguous spaces (or tabs) as one delimiter. May return pointer to
  * last char if field is not found. Equivalent to awk '{print $field}'.
@@ -264,12 +277,10 @@ const char *field_start(const char *p, int field)
 		/* skip spaces */
 		while (1) {
 			c = *(p++);
-			if (c > ' ')
-				break;
-			if (c == ' ')
-				continue;
 			if (!c) /* end of line */
 				return p-1;
+			if (c == ' ')
+				continue;
 			/* other char => new field */
 			break;
 		}
@@ -281,13 +292,31 @@ const char *field_start(const char *p, int field)
 
 		/* skip this field */
 		while (1) {
-			c = *(p++);
-			if (c == ' ')
+#if defined(HA_UNALIGNED_LE64)
+			unsigned long long l = *(unsigned long long *)p;
+			if (!has_zero64(l)) {
+				l ^= 0x2020202020202020;
+				l = has_zero64(l);
+				if (!l) {
+					p += 8;
+					continue;
+				}
+				/* there is at least one space, find it and
+				 * skip it now. The lowest byte in <l> with
+				 * a 0x80 is the right one, but checking for
+				 * it remains slower than testing each byte,
+				 * probably due to the numerous short fields.
+				 */
+				while (*(p++) != ' ')
+					;
 				break;
-			if (c > ' ')
-				continue;
+			}
+#endif
+			c = *(p++);
 			if (c == '\0')
 				return p - 1;
+			if (c == ' ')
+				break;
 		}
 	}
 #else
@@ -376,7 +405,7 @@ struct timer *insert_timer(struct eb_root *r, struct timer **alloc, int v)
 	struct eb32_node *n;
 
 	if (!t) {
-		t = calloc(1, sizeof(*t));
+		t = calloc(sizeof(*t), 1);
 		if (unlikely(!t)) {
 			fprintf(stderr, "%s: not enough memory\n", __FUNCTION__);
 			exit(1);
@@ -404,7 +433,7 @@ struct timer *insert_value(struct eb_root *r, struct timer **alloc, int v)
 	struct eb32_node *n;
 
 	if (!t) {
-		t = calloc(1, sizeof(*t));
+		t = calloc(sizeof(*t), 1);
 		if (unlikely(!t)) {
 			fprintf(stderr, "%s: not enough memory\n", __FUNCTION__);
 			exit(1);
@@ -450,22 +479,6 @@ int str2ic(const char *s)
 }
 
 
-/* Equivalent to strtoul with a length. */
-static inline unsigned int __strl2ui(const char *s, int len)
-{
-	unsigned int i = 0;
-	while (len-- > 0) {
-		i = i * 10 - '0';
-		i += (unsigned char)*s++;
-	}
-	return i;
-}
-
-unsigned int strl2ui(const char *s, int len)
-{
-	return __strl2ui(s, len);
-}
-
 /* Convert "[04/Dec/2008:09:49:40.555]" to an integer equivalent to the time of
  * the day in milliseconds. It returns -1 for all unparsable values. The parser
  * looks ugly but gcc emits far better code that way.
@@ -551,7 +564,8 @@ int convert_date_to_timestamp(const char *field)
 	d = mo = y = h = m = s = 0;
 	e = field;
 
-	c = *(e++); // remove '['
+	e++; // remove '['
+
 	/* day + '/' */
 	while (1) {
 		c = *(e++) - '0';
@@ -1136,7 +1150,7 @@ int main(int argc, char **argv)
 				ms = h % 1000; h = h / 1000;
 				s = h % 60; h = h / 60;
 				m = h % 60; h = h / 60;
-				printf("%02d:%02d:%02d.%03d %d %d %d\n", h, m, s, ms, last, d, t->count);
+				printf("%02u:%02u:%02u.%03u %d %u %u\n", h, m, s, ms, last, d, t->count);
 				lines_out++;
 				if (lines_max >= 0 && lines_out >= lines_max)
 					break;
@@ -1148,13 +1162,12 @@ int main(int argc, char **argv)
 		/* sort all timers */
 		for (f = 0; f < 5; f++) {
 			struct eb32_node *n;
-			int val;
 
-			val = 0;
 			n = eb32_first(&timers[f]);
 			while (n) {
 				int i;
 				double d;
+				int val;
 
 				t = container_of(n, struct timer, node);
 				last = n->key;
@@ -1195,7 +1208,7 @@ int main(int argc, char **argv)
 		for (step = 1; step <= 1000;) {
 			unsigned int thres = lines_out * (step / 1000.0);
 
-			printf("%3.1f %d ", step/10.0, thres);
+			printf("%3.1f %u ", step/10.0, thres);
 			for (f = 1; f < 5; f++) {
 				struct eb32_node *next;
 				while (cum[f] < thres) {
@@ -1224,7 +1237,7 @@ int main(int argc, char **argv)
 		n = eb32_first(&timers[0]);
 		while (n) {
 			t = container_of(n, struct timer, node);
-			printf("%d %d\n", n->key, t->count);
+			printf("%d %u\n", n->key, t->count);
 			lines_out++;
 			if (lines_max >= 0 && lines_out >= lines_max)
 				break;
@@ -1247,7 +1260,7 @@ int main(int argc, char **argv)
 			for (f = 0; f <= 5; f++)
 				tot_rq += srv->st_cnt[f];
 
-			printf("%s %d %d %d %d %d %d %d %d %.1f %d %d\n",
+			printf("%s %u %u %u %u %u %u %d %u %.1f %d %d\n",
 			       srv_node->key, srv->st_cnt[1], srv->st_cnt[2],
 			       srv->st_cnt[3], srv->st_cnt[4], srv->st_cnt[5], srv->st_cnt[0],
 			       tot_rq,
@@ -1264,7 +1277,7 @@ int main(int argc, char **argv)
 		n = eb32_first(&timers[0]);
 		while (n) {
 			t = container_of(n, struct timer, node);
-			printf("%c%c %d\n", (n->key >> 8), (n->key) & 255, t->count);
+			printf("%c%c %u\n", (n->key >> 8), (n->key) & 255, t->count);
 			lines_out++;
 			if (lines_max >= 0 && lines_out >= lines_max)
 				break;
@@ -1320,7 +1333,7 @@ int main(int argc, char **argv)
 		node = eb_last(&timers[0]);
 		while (node) {
 			ustat = container_of(node, struct url_stat, node.url.node);
-			printf("%d %d %llu %llu %llu %llu %llu %llu %s\n",
+			printf("%u %u %llu %llu %llu %llu %llu %llu %s\n",
 			       ustat->nb_req,
 			       ustat->nb_err,
 			       ustat->total_time,
