@@ -156,6 +156,7 @@ void show_conn_flags(unsigned int f)
 	SHOW_FLAG(f, CO_FL_SEND_PROXY);
 	SHOW_FLAG(f, CO_FL_WAIT_L6_CONN);
 	SHOW_FLAG(f, CO_FL_WAIT_L4_CONN);
+	SHOW_FLAG(f, CO_FL_FDLESS);
 	SHOW_FLAG(f, CO_FL_ERROR);
 	SHOW_FLAG(f, CO_FL_SOCK_WR_SH);
 	SHOW_FLAG(f, CO_FL_SOCK_RD_SH);
@@ -408,7 +409,14 @@ void show_strm_flags(unsigned int f)
 
 void usage_exit(const char *name)
 {
-	fprintf(stderr, "Usage: %s [ana|chn|conn|sc|si|sierr|strm|task|txn]* { [+-][0x]value* | - }\n", name);
+	int word, nbword;
+
+	fprintf(stderr, "Usage: %s [", name);
+
+	nbword = sizeof(show_as_words) / sizeof(*show_as_words);
+	for (word = 0; word < nbword; word++)
+		fprintf(stderr, "%s%s", word ? "|" : "", show_as_words[word]);
+	fprintf(stderr, "]* { [+-][0x]value* | - }\n");
 	exit(1);
 }
 
