@@ -28,11 +28,11 @@
 #include <haproxy/global.h>
 #include <haproxy/htx.h>
 #include <haproxy/stream.h>
-#include <haproxy/stream_interface-t.h>
 #include <haproxy/task.h>
 #include <haproxy/ticks.h>
 #include <haproxy/tools-t.h>
 
+struct stconn;
 
 /* perform minimal intializations, report 0 in case of error, 1 if OK. */
 int init_channel();
@@ -64,22 +64,22 @@ static inline struct stream *chn_strm(const struct channel *chn)
 		return LIST_ELEM(chn, struct stream *, req);
 }
 
-/* returns a pointer to the stream interface feeding the channel (producer) */
-static inline struct stream_interface *chn_prod(const struct channel *chn)
+/* returns a pointer to the stream connector feeding the channel (producer) */
+static inline struct stconn *chn_prod(const struct channel *chn)
 {
 	if (chn->flags & CF_ISRESP)
-		return &LIST_ELEM(chn, struct stream *, res)->si[1];
+		return LIST_ELEM(chn, struct stream *, res)->scb;
 	else
-		return &LIST_ELEM(chn, struct stream *, req)->si[0];
+		return LIST_ELEM(chn, struct stream *, req)->scf;
 }
 
-/* returns a pointer to the stream interface consuming the channel (producer) */
-static inline struct stream_interface *chn_cons(const struct channel *chn)
+/* returns a pointer to the stream connector consuming the channel (producer) */
+static inline struct stconn *chn_cons(const struct channel *chn)
 {
 	if (chn->flags & CF_ISRESP)
-		return &LIST_ELEM(chn, struct stream *, res)->si[0];
+		return LIST_ELEM(chn, struct stream *, res)->scf;
 	else
-		return &LIST_ELEM(chn, struct stream *, req)->si[1];
+		return LIST_ELEM(chn, struct stream *, req)->scb;
 }
 
 /* c_orig() : returns the pointer to the channel buffer's origin */
@@ -127,6 +127,7 @@ static inline size_t c_full(const struct channel *c)
 /* co_data() : returns the amount of output data in the channel's buffer */
 static inline size_t co_data(const struct channel *c)
 {
+	CHECK_IF_HOT(c->output > c_data(c));
 	return c->output;
 }
 
@@ -170,6 +171,7 @@ static inline char *c_ptr(const struct channel *c, ssize_t ofs)
 static inline void c_adv(struct channel *c, size_t adv)
 {
 	c->output += adv;
+	BUG_ON_HOT(c->output > c_data(c));
 }
 
 /* c_rew() : rewinds the channel's buffer by <adv> bytes, which means that the
@@ -179,6 +181,7 @@ static inline void c_adv(struct channel *c, size_t adv)
  */
 static inline void c_rew(struct channel *c, size_t adv)
 {
+	BUG_ON_HOT(c->output < adv);
 	c->output -= adv;
 }
 
@@ -191,6 +194,7 @@ static inline void c_realign_if_empty(struct channel *chn)
 /* Sets the amount of output for the channel */
 static inline void co_set_data(struct channel *c, size_t output)
 {
+	BUG_ON_HOT(output > c_data(c));
 	c->output = output;
 }
 
@@ -429,7 +433,7 @@ static inline int channel_is_rewritable(const struct channel *chn)
  */
 static inline int channel_may_send(const struct channel *chn)
 {
-	return chn_cons(chn)->state == SI_ST_EST;
+	return chn_cons(chn)->state == SC_ST_EST;
 }
 
 /* HTX version of channel_may_recv(). Returns non-zero if the channel can still
@@ -473,7 +477,7 @@ static inline int channel_htx_may_recv(const struct channel *chn, const struct h
  * are considered as available since they're supposed to leave the buffer. The
  * test is optimized to avoid as many operations as possible for the fast case
  * and to be used as an "if" condition. Just like channel_recv_limit(), we
- * never allow to overwrite the reserve until the output stream interface is
+ * never allow to overwrite the reserve until the output stream connector is
  * connected, otherwise we could spin on a POST with http-send-name-header.
  */
 static inline int channel_may_recv(const struct channel *chn)
@@ -932,6 +936,7 @@ static inline int32_t channel_htx_fwd_headers(struct channel *chn, struct htx *h
  */
 static inline void co_skip(struct channel *chn, int len)
 {
+	BUG_ON_HOT(len > chn->output);
 	b_del(&chn->buf, len);
 	chn->output -= len;
 	c_realign_if_empty(chn);
@@ -946,8 +951,10 @@ static inline void co_htx_skip(struct channel *chn, struct htx *htx, int len)
 	struct htx_ret htxret;
 
 	htxret = htx_drain(htx, len);
-	if (htxret.ret)
+	if (htxret.ret) {
+		BUG_ON_HOT(htxret.ret > chn->output);
 		chn->output -= htxret.ret;
+	}
 }
 
 /* Tries to copy chunk <chunk> into the channel's buffer after length controls.
@@ -999,19 +1006,6 @@ static inline int co_getchr(struct channel *chn)
 	return *co_head(chn);
 }
 
-/* Remove a block <blk> in a <htx> structure which is used by a channel <chn>
- * Update the channel output according to the size of the block removed
- * Return the size of the removed block*/
-static inline int32_t co_htx_remove_blk(struct channel *chn, struct htx *htx, struct htx_blk *blk)
-{
-	int32_t size = htx_get_blksz(blk);
-
-	htx_remove_blk(htx, blk);
-	co_set_data(chn, co_data(chn) - size);
-
-	return size;
-}
-
 #endif /* _HAPROXY_CHANNEL_H */
 
 /*
