@@ -16,7 +16,8 @@
 #define _HAPROXY_CHECKS_T_H
 
 #include <sys/time.h>
-#include <import/ebpttree.h>
+
+#include <import/ebtree-t.h>
 #include <import/ist.h>
 #include <haproxy/api-t.h>
 #include <haproxy/buf-t.h>
@@ -54,6 +55,7 @@ enum chk_result {
 #define CHK_ST_IN_ALLOC         0x0040  /* check blocked waiting for input buffer allocation */
 #define CHK_ST_OUT_ALLOC        0x0080  /* check blocked waiting for output buffer allocation */
 #define CHK_ST_CLOSE_CONN       0x0100  /* check is waiting that the connection gets closed */
+#define CHK_ST_PURGE            0x0200  /* check must be freed */
 
 /* check status */
 enum healthcheck_status {
@@ -148,7 +150,7 @@ struct check {
 	struct session *sess;			/* Health check session. */
 	struct vars vars;			/* Health check dynamic variables. */
 	struct xprt_ops *xprt;			/* transport layer operations for health checks */
-	struct conn_stream *cs;			/* conn_stream state for health checks */
+	struct stconn *sc;			/* stream connector used by health checks */
 	struct buffer bi, bo;			/* input and output buffers to send/recv check */
 	struct buffer_wait buf_wait;            /* Wait list for buffer allocation */
 	struct task *task;			/* the task associated to the health check processing, NULL if disabled */
@@ -174,7 +176,6 @@ struct check {
 	char **envp;				/* the environment to use if running a process-based check */
 	struct pid_list *curpid;		/* entry in pid_list used for current process-based test, or -1 if not in test */
 	struct sockaddr_storage addr;   	/* the address to check */
-	struct wait_event wait_list;            /* Waiting for I/O events */
 	char *sni;				/* Server name */
 	char *alpn_str;                         /* ALPN to use for checks */
 	int alpn_len;                           /* ALPN string length */
