@@ -21,7 +21,7 @@ int ha_cpuset_clr(struct hap_cpuset *set, int cpu);
 
 /* Bitwise and equivalent operation between <src> and <dst> stored in <dst>.
  */
-void ha_cpuset_and(struct hap_cpuset *dst, const struct hap_cpuset *src);
+void ha_cpuset_and(struct hap_cpuset *dst, struct hap_cpuset *src);
 
 /* Returns the count of set index in <set>.
  */
@@ -35,7 +35,7 @@ int ha_cpuset_ffs(const struct hap_cpuset *set);
 
 /* Copy <src> set into <dst>.
  */
-void ha_cpuset_assign(struct hap_cpuset *dst, const struct hap_cpuset *src);
+void ha_cpuset_assign(struct hap_cpuset *dst, struct hap_cpuset *src);
 
 /* Returns the biggest index plus one usable on the platform.
  */
