@@ -82,16 +82,6 @@ static inline unsigned long proc_mask(unsigned long mask)
 	return mask ? mask : 1;
 }
 
-/* handle 'tainted' status */
-enum tainted_flags {
-	TAINTED_CONFIG_EXP_KW_DECLARED = 0x1,
-	TAINTED_ACTION_EXP_EXECUTED    = 0x2,
-	TAINTED_CLI_EXPERT_MODE        = 0x4,
-	TAINTED_CLI_EXPERIMENTAL_MODE  = 0x8,
-};
-void mark_tainted(const enum tainted_flags flag);
-unsigned int get_tainted(void);
-
 extern unsigned int experimental_directives_allowed;
 
 struct cfg_keyword;
