@@ -61,8 +61,9 @@ struct qcc {
 
 		uint64_t ms_uni; /* max sub-ID of uni stream allowed for the peer */
 
-		uint64_t msd_bidi_l; /* initial max-stream-data on local streams */
-		uint64_t msd_bidi_r; /* initial max-stream-data on remote streams */
+		uint64_t msd_bidi_l; /* initial max-stream-data on local bidi streams */
+		uint64_t msd_bidi_r; /* initial max-stream-data on remote bidi streams */
+		uint64_t msd_uni_r; /* initial max-stream-data on remote uni streams */
 
 		uint64_t md; /* current max-data allowed for the peer */
 		uint64_t md_init; /* initial max-data */
@@ -73,8 +74,9 @@ struct qcc {
 	/* flow-control fields set by the peer which we must respect. */
 	struct {
 		uint64_t md; /* connection flow control limit updated on MAX_DATA frames reception */
-		uint64_t msd_bidi_l; /* initial max-stream-data for peer local streams */
-		uint64_t msd_bidi_r; /* initial max-stream-data for peer remote streams */
+		uint64_t msd_bidi_l; /* initial max-stream-data from peer on local bidi streams */
+		uint64_t msd_bidi_r; /* initial max-stream-data from peer on remote bidi streams */
+		uint64_t msd_uni_l; /* initial max-stream-data from peer on local uni streams */
 	} rfctl;
 
 	struct {
@@ -95,7 +97,6 @@ struct qcc {
 	struct list send_retry_list; /* list of qcs eligible to send retry */
 
 	struct wait_event wait_event;  /* To be used if we're waiting for I/Os */
-	struct wait_event *subs;
 
 	struct proxy *proxy;
 
@@ -182,12 +183,19 @@ struct qcs {
 	int start; /* base timestamp for http-request timeout */
 };
 
+/* Used as qcc_app_ops.close callback argument. */
+enum qcc_app_ops_close_side {
+	QCC_APP_OPS_CLOSE_SIDE_RD, /* Read channel closed (RESET_STREAM received). */
+	QCC_APP_OPS_CLOSE_SIDE_WR /* Write channel closed (STOP_SENDING received). */
+};
+
 /* QUIC application layer operations */
 struct qcc_app_ops {
 	int (*init)(struct qcc *qcc);
 	int (*attach)(struct qcs *qcs, void *conn_ctx);
 	ssize_t (*decode_qcs)(struct qcs *qcs, struct buffer *b, int fin);
 	size_t (*snd_buf)(struct qcs *qcs, struct htx *htx, size_t count);
+	int (*close)(struct qcs *qcs, enum qcc_app_ops_close_side side);
 	void (*detach)(struct qcs *qcs);
 	int (*finalize)(void *ctx);
 	void (*shutdown)(void *ctx);                    /* Close a connection. */
