@@ -6,13 +6,11 @@
 #error "Must define USE_OPENSSL"
 #endif
 
-#include <import/eb64tree.h>
-
 #include <haproxy/api.h>
 #include <haproxy/connection.h>
+#include <haproxy/list.h>
 #include <haproxy/mux_quic-t.h>
 #include <haproxy/stream.h>
-#include <haproxy/xprt_quic-t.h>
 
 struct qcs *qcc_init_stream_local(struct qcc *qcc, int bidi);
 struct buffer *qc_get_buf(struct qcs *qcs, struct buffer *bptr);
@@ -122,7 +120,7 @@ static inline struct stconn *qc_attach_sc(struct qcs *qcs, struct buffer *buf)
 	 * it to sedesc. See <qcs_wait_http_req> for more info.
 	 */
 	BUG_ON_HOT(!LIST_INLIST(&qcs->el_opening));
-	LIST_DELETE(&qcs->el_opening);
+	LIST_DEL_INIT(&qcs->el_opening);
 
 	return qcs->sd->sc;
 }
