@@ -22,7 +22,7 @@
 #ifndef _HAPROXY_PATTERN_T_H
 #define _HAPROXY_PATTERN_T_H
 
-#include <import/ebmbtree.h>
+#include <import/ebtree-t.h>
 
 #include <haproxy/api-t.h>
 #include <haproxy/regex-t.h>
@@ -110,6 +110,7 @@ struct pat_ref {
 	unsigned int next_gen; /* next generation number (insertions use this one) */
 	int unique_id; /* Each pattern reference have unique id. */
 	unsigned long long revision; /* updated for each update */
+	unsigned long long entry_cnt; /* the total number of entries */
 	__decl_thread(HA_SPINLOCK_T lock); /* Lock used to protect pat ref elements */
 };
 
@@ -199,7 +200,6 @@ struct pattern_expr {
 	struct eb_root pattern_tree;  /* may be used for lookup in large datasets */
 	struct eb_root pattern_tree_2;  /* may be used for different types */
 	int mflags;                     /* flags relative to the parsing or matching method. */
-	uint32_t refcount;            /* refcount used to know if the expr can be deleted or not */
 	__decl_thread(HA_RWLOCK_T lock);               /* lock used to protect patterns */
 };
 
@@ -209,6 +209,7 @@ struct pattern_expr {
  */
 struct pattern_expr_list {
 	struct list list; /* Used for chaining pattern_expr in pattern_head. */
+	int do_free;
 	struct pattern_expr *expr; /* The used expr. */
 };
 
