@@ -25,7 +25,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
-#include <import/eb32tree.h>
+#include <import/ebtree-t.h>
 #include <haproxy/api-t.h>
 
 /* some pointer types referenced below */
@@ -48,6 +48,13 @@ struct connection;
 # error "Can't build on the target system, AF_CUST_MAX overflow"
 #endif
 
+/* socket-level protocol types, used for protocol selection */
+enum proto_type {
+	PROTO_TYPE_STREAM,      /* streaming protocol (like TCP) */
+	PROTO_TYPE_DGRAM,       /* datagram protocol (like UDP) */
+	PROTO_NUM_TYPES         /* must be the last one */
+};
+
 /* max length of a protocol name, including trailing zero */
 #define PROTO_NAME_LEN 16
 
@@ -68,8 +75,8 @@ struct proto_fam {
 	int l3_addrlen;					/* layer3 address length, used by hashes */
 	int (*addrcmp)(const struct sockaddr_storage *, const struct sockaddr_storage *); /* compare addresses (like memcmp) */
 	int (*bind)(struct receiver *rx, char **errmsg); /* bind a receiver */
-	int (*get_src)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve src addr */
-	int (*get_dst)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve dst addr */
+	int (*get_src)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve connection's src addr */
+	int (*get_dst)(int fd, struct sockaddr *, socklen_t, int dir); /* syscall used to retrieve connection's dst addr */
 	void (*set_port)(struct sockaddr_storage *, int port);  /* set the port on the address; NULL if not implemented */
 };
 
@@ -82,7 +89,8 @@ struct proto_fam {
 struct protocol {
 	char name[PROTO_NAME_LEN];			/* protocol name, zero-terminated */
 	struct proto_fam *fam;                          /* protocol family */
-	int ctrl_type;                                  /* control layer type (SOCK_STREAM/SOCK_DGRAM) */
+	int xprt_type;                                  /* transport layer type (PROTO_TYPE_STREAM/PROTO_TYPE_DGRAM) */
+	enum proto_type proto_type;                     /* protocol type at the socket layer (PROTO_TYPE_*) */
 	int sock_type;					/* socket type, as passed to socket()     */
 	int sock_prot;					/* socket protocol, as passed to socket() */
 
@@ -95,6 +103,7 @@ struct protocol {
 	int (*suspend)(struct listener *l);             /* try to suspend the listener */
 	int (*resume)(struct listener *l);              /* try to resume a suspended listener */
 	struct connection *(*accept_conn)(struct listener *l, int *status); /* accept a new connection */
+
 	/* functions acting on connections */
 	void (*ctrl_init)(struct connection *);         /* completes initialization of the connection */
 	void (*ctrl_close)(struct connection *);        /* completes release of the connection */
@@ -102,6 +111,8 @@ struct protocol {
 	int (*drain)(struct connection *);              /* drain pending data; 0=failed, >0=success */
 	int (*check_events)(struct connection *conn, int event_type);  /* subscribe to socket events */
 	void (*ignore_events)(struct connection *conn, int event_type);  /* unsubscribe from socket events */
+	int (*get_src)(struct connection *conn, struct sockaddr *, socklen_t); /* retrieve connection's source address; -1=fail */
+	int (*get_dst)(struct connection *conn, struct sockaddr *, socklen_t); /* retrieve connection's dest address; -1=fail */
 
 	/* functions acting on the receiver */
 	int (*rx_suspend)(struct receiver *rx);         /* temporarily suspend this receiver for a soft restart */
