@@ -65,11 +65,7 @@ struct receiver {
 	struct rx_settings *settings;    /* points to the settings used by this receiver */
 	struct list proto_list;          /* list in the protocol header */
 #ifdef USE_QUIC
-	struct qring **tx_qrings;         /* Array of rings (one by thread) */
-	struct mt_list tx_qring_list;    /* The same as ->tx_qrings but arranged in a list */
-
-	struct rxbuf **rxbufs;           /* Array of buffers for RX (one by thread) */
-	struct mt_list rxbuf_list;       /* The same as ->rxbufs but arranged in a list */
+	struct mt_list rxbuf_list;       /* list of buffers to receive and dispatch QUIC datagrams. */
 #endif
 	/* warning: this struct is huge, keep it at the bottom */
 	struct sockaddr_storage addr;    /* the address the socket is bound to */
