@@ -402,9 +402,9 @@ static inline void _tasklet_wakeup_on(struct tasklet *tl, int thr, const char *f
 	tl->debug.caller_idx = !tl->debug.caller_idx;
 	tl->debug.caller_file[tl->debug.caller_idx] = file;
 	tl->debug.caller_line[tl->debug.caller_idx] = line;
-	if (task_profiling_mask & tid_bit)
-		tl->call_date = now_mono_time();
 #endif
+	if (task_profiling_mask & tid_bit)
+		tl->wake_date = now_mono_time();
 	__tasklet_wakeup_on(tl, thr);
 }
 
@@ -427,12 +427,11 @@ static inline void _tasklet_wakeup_on(struct tasklet *tl, int thr, const char *f
 #define task_instant_wakeup(t, f) _task_instant_wakeup(t, f, __FILE__, __LINE__)
 static inline void _task_instant_wakeup(struct task *t, unsigned int f, const char *file, int line)
 {
-	struct tasklet *tl = (struct tasklet *)t;
 	int thr = my_ffsl(t->thread_mask) - 1;
 	unsigned int state;
 
 	/* first, let's update the task's state with the wakeup condition */
-	state = _HA_ATOMIC_OR_FETCH(&tl->state, f);
+	state = _HA_ATOMIC_OR_FETCH(&t->state, f);
 
 	/* next we need to make sure the task was not/will not be added to the
 	 * run queue because the tasklet list's mt_list uses the same storage
@@ -442,21 +441,21 @@ static inline void _task_instant_wakeup(struct task *t, unsigned int f, const ch
 		/* do nothing if someone else already added it */
 		if (state & (TASK_QUEUED|TASK_RUNNING))
 			return;
-	} while (!_HA_ATOMIC_CAS(&tl->state, &state, state | TASK_QUEUED));
+	} while (!_HA_ATOMIC_CAS(&t->state, &state, state | TASK_QUEUED));
 
 	BUG_ON_HOT(task_in_rq(t));
 
 	/* at this point we're the first ones to add this task to the list */
 #ifdef DEBUG_TASK
-	if ((unsigned int)tl->debug.caller_idx > 1)
+	if ((unsigned int)t->debug.caller_idx > 1)
 		ABORT_NOW();
-	tl->debug.caller_idx = !tl->debug.caller_idx;
-	tl->debug.caller_file[tl->debug.caller_idx] = file;
-	tl->debug.caller_line[tl->debug.caller_idx] = line;
-	if (task_profiling_mask & tid_bit)
-		tl->call_date = now_mono_time();
+	t->debug.caller_idx = !t->debug.caller_idx;
+	t->debug.caller_file[t->debug.caller_idx] = file;
+	t->debug.caller_line[t->debug.caller_idx] = line;
 #endif
-	__tasklet_wakeup_on(tl, thr);
+	if (task_profiling_mask & tid_bit)
+		t->wake_date = now_mono_time();
+	__tasklet_wakeup_on((struct tasklet *)t, thr);
 }
 
 /* schedules tasklet <tl> to run immediately after the current one is done
@@ -487,9 +486,9 @@ static inline struct list *_tasklet_wakeup_after(struct list *head, struct taskl
 	tl->debug.caller_idx = !tl->debug.caller_idx;
 	tl->debug.caller_file[tl->debug.caller_idx] = file;
 	tl->debug.caller_line[tl->debug.caller_idx] = line;
-	if (th_ctx->flags & TH_FL_TASK_PROFILING)
-		tl->call_date = now_mono_time();
 #endif
+	if (task_profiling_mask & tid_bit)
+		tl->wake_date = now_mono_time();
 	return __tasklet_wakeup_after(head, tl);
 }
 
@@ -537,7 +536,7 @@ static inline struct task *task_init(struct task *t, unsigned long thread_mask)
 		t->state |= TASK_SHARED_WQ;
 	t->nice = 0;
 	t->calls = 0;
-	t->call_date = 0;
+	t->wake_date = 0;
 	t->cpu_time = 0;
 	t->lat_time = 0;
 	t->expire = TICK_ETERNITY;
@@ -557,6 +556,7 @@ static inline void tasklet_init(struct tasklet *t)
 	t->state = TASK_F_TASKLET;
 	t->process = NULL;
 	t->tid = -1;
+	t->wake_date = 0;
 #ifdef DEBUG_TASK
 	t->debug.caller_idx = 0;
 #endif
