@@ -1,6 +1,6 @@
 /*
  * include/haproxy/time.h
- * Time calculation functions and macros.
+ * timeval-based time calculation functions and macros.
  *
  * Copyright (C) 2000-2020 Willy Tarreau - w@1wt.eu
  *
@@ -23,43 +23,10 @@
 #define _HAPROXY_TIME_H
 
 #include <sys/time.h>
-#include <time.h>
 #include <haproxy/api.h>
-#include <haproxy/thread.h>
-
-/* eternity when exprimed in timeval */
-#ifndef TV_ETERNITY
-#define TV_ETERNITY     (~0UL)
-#endif
-
-/* eternity when exprimed in ms */
-#ifndef TV_ETERNITY_MS
-#define TV_ETERNITY_MS  (-1)
-#endif
 
 #define TIME_ETERNITY   (TV_ETERNITY_MS)
 
-/* we want to be able to detect time jumps. Fix the maximum wait time to a low
- * value so that we know the time has changed if we wait longer.
- */
-#define MAX_DELAY_MS    60000
-
-
-/* returns the lowest delay amongst <old> and <new>, and respects TIME_ETERNITY */
-#define MINTIME(old, new)	(((new)<0)?(old):(((old)<0||(new)<(old))?(new):(old)))
-#define SETNOW(a)		(*a=now)
-
-extern THREAD_LOCAL unsigned int   now_ms;           /* internal date in milliseconds (may wrap) */
-extern THREAD_LOCAL unsigned int   samp_time;        /* total elapsed time over current sample */
-extern THREAD_LOCAL unsigned int   idle_time;        /* total idle time over current sample */
-extern THREAD_LOCAL struct timeval now;              /* internal date is a monotonic function of real clock */
-extern THREAD_LOCAL struct timeval date;             /* the real current date */
-extern struct timeval start_date;                    /* the process's start date */
-extern struct timeval ready_date;                    /* date when the process was considered ready */
-extern THREAD_LOCAL struct timeval before_poll;      /* system date before calling poll() */
-extern THREAD_LOCAL struct timeval after_poll;       /* system date after leaving poll() */
-extern volatile unsigned long long global_now;
-extern volatile unsigned int global_now_ms;
 
 
 /**** exported functions *************************************************/
@@ -80,30 +47,10 @@ int tv_ms_cmp(const struct timeval *tv1, const struct timeval *tv2);
  */
 int tv_ms_cmp2(const struct timeval *tv1, const struct timeval *tv2);
 
-/* tv_udpate_date: sets <date> to system time, and sets <now> to something as
- * close as possible to real time, following a monotonic function. The main
- * principle consists in detecting backwards and forwards time jumps and adjust
- * an offset to correct them. This function should be called only once after
- * each poll. The poll's timeout should be passed in <max_wait>, and the return
- * value in <interrupted> (a non-zero value means that we have not expired the
- * timeout).
- */
-void tv_update_date(int max_wait, int interrupted);
-void tv_init_process_date();
-void tv_init_thread_date();
-
-char *timeofday_as_iso_us(int pad);
 
 /**** general purpose functions and macros *******************************/
 
 
-/* tv_now: sets <tv> to the current time */
-static inline struct timeval *tv_now(struct timeval *tv)
-{
-	gettimeofday(tv, NULL);
-	return tv;
-}
-
 /*
  * sets a struct timeval to its highest value so that it can never happen
  * note that only tv_usec is necessary to detect it since a tv_usec > 999999
@@ -517,107 +464,6 @@ static inline struct timeval *__tv_ms_add(struct timeval *tv, const struct timev
         tv1;                       \
 })
 
-/* returns the system's monotonic time in nanoseconds if supported, otherwise zero */
-static inline uint64_t now_mono_time()
-{
-#if (_POSIX_TIMERS > 0) && defined(_POSIX_MONOTONIC_CLOCK)
-	struct timespec ts;
-	clock_gettime(CLOCK_MONOTONIC, &ts);
-	return ts.tv_sec * 1000000000ULL + ts.tv_nsec;
-#else
-	return 0;
-#endif
-}
-
-/* returns the current thread's cumulated CPU time in nanoseconds if supported, otherwise zero */
-static inline uint64_t now_cpu_time()
-{
-#if (_POSIX_TIMERS > 0) && defined(_POSIX_THREAD_CPUTIME)
-	struct timespec ts;
-	clock_gettime(CLOCK_THREAD_CPUTIME_ID, &ts);
-	return ts.tv_sec * 1000000000ULL + ts.tv_nsec;
-#else
-	return 0;
-#endif
-}
-
-/* returns another thread's cumulated CPU time in nanoseconds if supported, otherwise zero */
-static inline uint64_t now_cpu_time_thread(const struct thread_info *thr)
-{
-#if (_POSIX_TIMERS > 0) && defined(_POSIX_THREAD_CPUTIME)
-	struct timespec ts;
-	clock_gettime(thr->clock_id, &ts);
-	return ts.tv_sec * 1000000000ULL + ts.tv_nsec;
-#else
-	return 0;
-#endif
-}
-
-/* Update the idle time value twice a second, to be called after
- * tv_update_date() when called after poll(). It relies on <before_poll> to be
- * updated to the system time before calling poll().
- */
-static inline void measure_idle()
-{
-	/* Let's compute the idle to work ratio. We worked between after_poll
-	 * and before_poll, and slept between before_poll and date. The idle_pct
-	 * is updated at most twice every second. Note that the current second
-	 * rarely changes so we avoid a multiply when not needed.
-	 */
-	int delta;
-
-	if ((delta = date.tv_sec - before_poll.tv_sec))
-		delta *= 1000000;
-	idle_time += delta + (date.tv_usec - before_poll.tv_usec);
-
-	if ((delta = date.tv_sec - after_poll.tv_sec))
-		delta *= 1000000;
-	samp_time += delta + (date.tv_usec - after_poll.tv_usec);
-
-	after_poll.tv_sec = date.tv_sec; after_poll.tv_usec = date.tv_usec;
-	if (samp_time < 500000)
-		return;
-
-	HA_ATOMIC_STORE(&ti->idle_pct, (100ULL * idle_time + samp_time / 2) / samp_time);
-	idle_time = samp_time = 0;
-}
-
-/* report the average CPU idle percentage over all running threads, between 0 and 100 */
-static inline uint report_idle()
-{
-	uint total = 0;
-	uint rthr = 0;
-	uint thr;
-
-	for (thr = 0; thr < MAX_THREADS; thr++) {
-		if (!(all_threads_mask & (1UL << thr)))
-			continue;
-		total += HA_ATOMIC_LOAD(&ha_thread_info[thr].idle_pct);
-		rthr++;
-	}
-	return rthr ? total / rthr : 0;
-}
-
-/* Collect date and time information before calling poll(). This will be used
- * to count the run time of the past loop and the sleep time of the next poll.
- */
-static inline void tv_entering_poll()
-{
-	gettimeofday(&before_poll, NULL);
-}
-
-/* Collect date and time information after leaving poll(). <timeout> must be
- * set to the maximum sleep time passed to poll (in milliseconds), and
- * <interrupted> must be zero if the poller reached the timeout or non-zero
- * otherwise, which generally is provided by the poller's return value.
- */
-static inline void tv_leaving_poll(int timeout, int interrupted)
-{
-	measure_idle();
-	ti->prev_cpu_time  = now_cpu_time();
-	ti->prev_mono_time = now_mono_time();
-}
-
 #endif /* _HAPROXY_TIME_H */
 
 /*
