@@ -33,13 +33,13 @@
 #ifdef  CONFIG_PRODUCT_BRANCH
 #define PRODUCT_BRANCH    CONFIG_PRODUCT_BRANCH
 #else
-#define PRODUCT_BRANCH   "2.3"
+#define PRODUCT_BRANCH   "2.6"
 #endif
 
 #ifdef  CONFIG_PRODUCT_STATUS
 #define PRODUCT_STATUS    CONFIG_PRODUCT_STATUS
 #else
-#define PRODUCT_STATUS   "Status: End of life - please upgrade to branch 2.4."
+#define PRODUCT_STATUS   "Status: long-term supported branch - will stop receiving fixes around Q2 2027."
 #endif
 
 #ifdef CONFIG_PRODUCT_URL_BUGS
