@@ -101,7 +101,6 @@ struct acl_keyword *find_acl_kw(const char *kw)
 static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 {
 	struct arg *arg;
-	int unresolved = 0;
 
 	pattern_prune(&expr->pat);
 
@@ -110,7 +109,6 @@ static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 			break;
 		if (arg->type == ARGT_STR || arg->unresolved) {
 			chunk_destroy(&arg->data.str);
-			unresolved |= arg->unresolved;
 			arg->unresolved = 0;
 		}
 	}
@@ -756,7 +754,7 @@ const struct {
 } default_acl_list[] = {
 	{ .name = "TRUE",           .expr = {"always_true",""}},
 	{ .name = "FALSE",          .expr = {"always_false",""}},
-	{ .name = "LOCALHOST",      .expr = {"src","127.0.0.1/8",""}},
+	{ .name = "LOCALHOST",      .expr = {"src","127.0.0.1/8","::1",""}},
 	{ .name = "HTTP",           .expr = {"req.proto_http",""}},
 	{ .name = "HTTP_1.0",       .expr = {"req.ver","1.0",""}},
 	{ .name = "HTTP_1.1",       .expr = {"req.ver","1.1",""}},
@@ -1310,6 +1308,35 @@ int init_acl()
 	return err;
 }
 
+/* dump known ACL keywords on stdout */
+void acl_dump_kwd(void)
+{
+	struct acl_kw_list *kwl;
+	const struct acl_keyword *kwp, *kw;
+	const char *name;
+	int index;
+
+	for (kw = kwp = NULL;; kwp = kw) {
+		list_for_each_entry(kwl, &acl_keywords.list, list) {
+			for (index = 0; kwl->kw[index].kw != NULL; index++) {
+				if (strordered(kwp ? kwp->kw : NULL,
+					       kwl->kw[index].kw,
+					       kw != kwp ? kw->kw : NULL))
+					kw = &kwl->kw[index];
+			}
+		}
+
+		if (kw == kwp)
+			break;
+
+		name = kw->fetch_kw;
+		if (!name)
+			name = kw->kw;
+
+		printf("%s = %s -m %s\n", kw->kw, name, pat_match_names[kw->match_type]);
+	}
+}
+
 void free_acl_cond(struct acl_cond *cond)
 {
 	struct acl_term_suite *suite, *suiteb;
