@@ -82,10 +82,8 @@ static int alloc_trash_buffers_per_thread()
 static void free_trash_buffers_per_thread()
 {
 	chunk_destroy(&trash);
-	free(trash_buf2);
-	free(trash_buf1);
-	trash_buf2 = NULL;
-	trash_buf1 = NULL;
+	ha_free(&trash_buf2);
+	ha_free(&trash_buf1);
 }
 
 /* Initialize the trash buffers. It returns 0 if an error occurred. */
@@ -152,19 +150,15 @@ int chunk_printf(struct buffer *chk, const char *fmt, ...)
 int chunk_appendf(struct buffer *chk, const char *fmt, ...)
 {
 	va_list argp;
-	size_t room;
 	int ret;
 
 	if (!chk->area || !chk->size)
 		return 0;
 
-	room = chk->size - chk->data;
-	if (!room)
-		return chk->data;
-
 	va_start(argp, fmt);
-	ret = vsnprintf(chk->area + chk->data, room, fmt, argp);
-	if (ret >= room)
+	ret = vsnprintf(chk->area + chk->data, chk->size - chk->data, fmt,
+			argp);
+	if (ret >= chk->size - chk->data)
 		/* do not copy anything in case of truncation */
 		chk->area[chk->data] = 0;
 	else
@@ -313,6 +307,7 @@ int chunk_strcasecmp(const struct buffer *chk, const char *str)
 
 REGISTER_PER_THREAD_ALLOC(alloc_trash_buffers_per_thread);
 REGISTER_PER_THREAD_FREE(free_trash_buffers_per_thread);
+REGISTER_POST_DEINIT(free_trash_buffers_per_thread);
 
 /*
  * Local variables:
