@@ -356,7 +356,7 @@ static int cli_parse_show_startup_logs(char **args, char *payload, struct appctx
 	if (!startup_logs)
 		return cli_msg(appctx, LOG_INFO, "\n"); // nothing to print
 
-	return ring_attach_cli(startup_logs, appctx);
+	return ring_attach_cli(startup_logs, appctx, 0);
 }
 
 /* register cli keywords */
@@ -375,4 +375,6 @@ static void deinit_errors_buffers()
 	ha_free(&usermsgs_ctx.str.area);
 }
 
+/* errors might be used in threads and even before forking, thus 2 deinit */
 REGISTER_PER_THREAD_FREE(deinit_errors_buffers);
+REGISTER_POST_DEINIT(deinit_errors_buffers);
