@@ -219,8 +219,8 @@ static void _do_poll(struct poller *p, int exp, int wake)
 
 	for (count = 0; status > 0 && count < nbfd; count++) {
 		unsigned int n;
-		int ret;
 		int e = poll_events[count].revents;
+
 		fd = poll_events[count].fd;
 
 		if ((e & POLLRDHUP) && !(cur_poller.flags & HAP_POLL_F_RDHUP))
@@ -241,13 +241,7 @@ static void _do_poll(struct poller *p, int exp, int wake)
 		    ((e & POLLHUP)   ? FD_EV_SHUT_RW : 0) |
 		    ((e & POLLERR)   ? FD_EV_ERR_RW  : 0);
 
-		ret = fd_update_events(fd, n);
-
-		if (ret == FD_UPDT_MIGRATED) {
-			/* FD was migrated, let's stop polling it */
-			if (!HA_ATOMIC_BTS(&fdtab[fd].update_mask, tid))
-				fd_updt[fd_nbupdt++] = fd;
-		}
+		fd_update_events(fd, n);
 	}
 }
 
