@@ -31,6 +31,7 @@
 
 #include <haproxy/api.h>
 #include <haproxy/hpack-huff.h>
+#include <haproxy/net_helper.h>
 
 struct huff {
 	uint32_t c; /* code point */
@@ -1439,14 +1440,11 @@ int huff_dec(const uint8_t *huff, int hlen, char *out, int olen)
 		while (shift >= 32) {
 			curr = next;
 
-			/* read up to 4 bytes into next. FIXME: this should
-			 * later be optimized to perform a single 32-bit big
-			 * endian read when unaligned accesses are possible.
-			 */
+			/* read up to 4 bytes into next */
 			next = 0;
 
 			if (huff + 4 <= huff_end) {
-				next = (huff[0] << 24) + (huff[1] << 16) + (huff[2] <<  8) + huff[3];
+				next = read_n32(huff);
 				huff += 4;
 			}
 			else {
@@ -1454,10 +1452,10 @@ int huff_dec(const uint8_t *huff, int hlen, char *out, int olen)
 				 * distinguish shifted bits from a really inserted
 				 * EOS.
 				 */
-				next =  (((huff + 0 < huff_end) ? huff[0] : 0x00) << 24) +
-					(((huff + 1 < huff_end) ? huff[1] : 0x00) << 16) +
-					(((huff + 2 < huff_end) ? huff[2] : 0x00) <<  8) +
-					((huff + 3 < huff_end) ? huff[3] : 0x00);
+				next =  (((huff + 0 < huff_end) ? (uint32_t)huff[0] : 0x00) << 24) +
+					(((huff + 1 < huff_end) ? (uint32_t)huff[1] : 0x00) << 16) +
+					(((huff + 2 < huff_end) ? (uint32_t)huff[2] : 0x00) <<  8) +
+					 ((huff + 3 < huff_end) ? (uint32_t)huff[3] : 0x00);
 				huff = huff_end;
 			}
 
