@@ -423,6 +423,16 @@ int httpclient_req_xfer(struct httpclient *hc, struct ist src, int end)
 
 	/* if we copied all the data and the end flag is set */
 	if ((istlen(src) == ret) && end) {
+		/* no more data are expected. If the HTX buffer is empty, be
+		 * sure to add something (EOT block in this case) to have
+		 * something to send. It is important to be sure the EOM flags
+		 * will be handled by the endpoint. Because the message is
+		 * empty, this should not fail. Otherwise it is an error
+		 */
+		if (htx_is_empty(htx)) {
+			if (!htx_add_endof(htx, HTX_BLK_EOT))
+				goto error;
+		}
 		htx->flags |= HTX_FL_EOM;
 	}
 	htx_to_buf(htx, &hc->req.buf);
@@ -468,10 +478,10 @@ int httpclient_set_dst(struct httpclient *hc, const char *dst)
 }
 
 /*
- * Return a split URL in <scheme>, <host>, <port>
+ * Split <url> in <scheme>, <host>, <port>
  */
-static void httpclient_spliturl(struct ist url, enum http_scheme *scheme,
-                                struct ist *host, int *port)
+static int httpclient_spliturl(struct ist url, enum http_scheme *scheme,
+                               struct ist *host, int *port)
 {
 	enum http_scheme scheme_tmp = SCH_HTTP;
 	int port_tmp = 0;
@@ -481,6 +491,9 @@ static void httpclient_spliturl(struct ist url, enum http_scheme *scheme,
 
 	parser = http_uri_parser_init(url);
 	scheme_ist = http_parse_scheme(&parser);
+	if (!isttest(scheme_ist)) {
+		return 0;
+	}
 
 	if (isteqi(scheme_ist, ist("http://"))){
 		scheme_tmp = SCH_HTTP;
@@ -491,6 +504,9 @@ static void httpclient_spliturl(struct ist url, enum http_scheme *scheme,
 	}
 
 	authority_ist = http_parse_authority(&parser, 1);
+	if (!isttest(authority_ist)) {
+		return 0;
+	}
 	p = end = istend(authority_ist);
 
 	/* look for a port at the end of the authority */
@@ -513,6 +529,7 @@ static void httpclient_spliturl(struct ist url, enum http_scheme *scheme,
 	if (port)
 		*port = port_tmp;
 
+	return 1;
 }
 
 /*
@@ -567,8 +584,10 @@ void httpclient_stop_and_destroy(struct httpclient *hc)
 	if (hc->flags & HTTPCLIENT_FS_ENDED || !(hc->flags & HTTPCLIENT_FS_STARTED)) {
 		httpclient_destroy(hc);
 	} else {
-	/* if the client wasn't stopped, ask for a stop and destroy */
+		/* if the client wasn't stopped, ask for a stop and destroy */
 		hc->flags |= (HTTPCLIENT_FA_AUTOKILL | HTTPCLIENT_FA_STOP);
+		/* the calling applet doesn't exist anymore */
+		hc->caller = NULL;
 		if (hc->appctx)
 			appctx_wakeup(hc->appctx);
 	}
@@ -644,6 +663,11 @@ static void httpclient_applet_io_handler(struct appctx *appctx)
 	uint32_t sz;
 	int ret;
 
+	/* The IO handler could be called after the release, so we need to
+	 * check if hc is still there to run the IO handler */
+	if (!hc)
+		return;
+
 	while (1) {
 
 		/* required to stop */
@@ -949,7 +973,8 @@ static int httpclient_applet_init(struct appctx *appctx)
 
 
 	/* parse the URL and  */
-	httpclient_spliturl(hc->req.url, &scheme, &host, &port);
+	if (!httpclient_spliturl(hc->req.url, &scheme, &host, &port))
+		goto out_error;
 
 	if (hc->dst) {
 		/* if httpclient_set_dst() was used, sets the alternative address */
@@ -1050,6 +1075,10 @@ static void httpclient_applet_release(struct appctx *appctx)
 		httpclient_destroy(hc);
 	}
 
+	/* be sure not to use this ptr anymore if the IO handler is called a
+	 * last time */
+	appctx->svcctx = NULL;
+
 	return;
 }
 
@@ -1070,7 +1099,7 @@ static int httpclient_resolve_init()
 	char *do_resolve = NULL;
 	char *http_rules[][11] = {
 	       { "set-var(txn.hc_ip)", "dst", "" },
-	       { do_resolve, "hdr(Host),lower", "if", "{", "var(txn.hc_ip)", "-m", "ip", "0.0.0.0", "}", "" },
+	       { do_resolve, "hdr(Host),host_only", "if", "{", "var(txn.hc_ip)", "-m", "ip", "0.0.0.0", "}", "" },
 	       { "return", "status", "503", "if", "{", "var(txn.hc_ip)", "-m", "ip", "0.0.0.0", "}", "" },
 	       { "capture", "var(txn.hc_ip)", "len", "40", "" },
 	       { "set-dst", "var(txn.hc_ip)", "" },
@@ -1150,6 +1179,7 @@ static int httpclient_precheck()
 		goto err;
 	}
 
+	srv_settings_cpy(httpclient_srv_raw, &httpclient_proxy->defsrv, 0);
 	httpclient_srv_raw->iweight = 0;
 	httpclient_srv_raw->uweight = 0;
 	httpclient_srv_raw->xprt = xprt_get(XPRT_RAW);
@@ -1169,6 +1199,7 @@ static int httpclient_precheck()
 		err_code |= ERR_ALERT | ERR_FATAL;
 		goto err;
 	}
+	srv_settings_cpy(httpclient_srv_ssl, &httpclient_proxy->defsrv, 0);
 	httpclient_srv_ssl->iweight = 0;
 	httpclient_srv_ssl->uweight = 0;
 	httpclient_srv_ssl->xprt = xprt_get(XPRT_SSL);
@@ -1185,11 +1216,8 @@ static int httpclient_precheck()
 	/* if the verify is required, try to load the system CA */
 	if (httpclient_ssl_verify == SSL_SOCK_VERIFY_REQUIRED) {
 
-		if (!httpclient_ssl_ca_file)
-			httpclient_ssl_ca_file = strdup("@system-ca");
-
-		httpclient_srv_ssl->ssl_ctx.ca_file = httpclient_ssl_ca_file;
-		if (!ssl_store_load_locations_file(httpclient_srv_ssl->ssl_ctx.ca_file, 1, CAFILE_CERT)) {
+		httpclient_srv_ssl->ssl_ctx.ca_file = strdup(httpclient_ssl_ca_file ? httpclient_ssl_ca_file : "@system-ca");
+		if (!__ssl_store_load_locations_file(httpclient_srv_ssl->ssl_ctx.ca_file, 1, CAFILE_CERT, !hard_error_ssl)) {
 			/* if we failed to load the ca-file, only quits in
 			 * error with hard_error, otherwise just disable the
 			 * feature. */
