@@ -50,6 +50,21 @@ struct action_kw_list http_after_res_keywords = {
        .list = LIST_HEAD_INIT(http_after_res_keywords.list)
 };
 
+void http_req_keywords_register(struct action_kw_list *kw_list)
+{
+	LIST_APPEND(&http_req_keywords.list, &kw_list->list);
+}
+
+void http_res_keywords_register(struct action_kw_list *kw_list)
+{
+	LIST_APPEND(&http_res_keywords.list, &kw_list->list);
+}
+
+void http_after_res_keywords_register(struct action_kw_list *kw_list)
+{
+	LIST_APPEND(&http_after_res_keywords.list, &kw_list->list);
+}
+
 /*
  * Return the struct http_req_action_kw associated to a keyword.
  */
@@ -84,7 +99,7 @@ struct act_rule *parse_http_req_cond(const char **args, const char *file, int li
 	rule = new_act_rule(ACT_F_HTTP_REQ, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
-		goto out_err;
+		goto out;
 	}
 
 	if (((custom = action_http_req_custom(args[0])) != NULL)) {
@@ -148,7 +163,8 @@ struct act_rule *parse_http_req_cond(const char **args, const char *file, int li
 
 	return rule;
  out_err:
-	free(rule);
+	free_act_rule(rule);
+ out:
 	return NULL;
 }
 
@@ -162,7 +178,7 @@ struct act_rule *parse_http_res_cond(const char **args, const char *file, int li
 	rule = new_act_rule(ACT_F_HTTP_RES, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
-		goto out_err;
+		goto out;
 	}
 
 	if (((custom = action_http_res_custom(args[0])) != NULL)) {
@@ -226,7 +242,8 @@ struct act_rule *parse_http_res_cond(const char **args, const char *file, int li
 
 	return rule;
  out_err:
-	free(rule);
+	free_act_rule(rule);
+ out:
 	return NULL;
 }
 
@@ -241,7 +258,7 @@ struct act_rule *parse_http_after_res_cond(const char **args, const char *file,
 	rule = new_act_rule(ACT_F_HTTP_RES, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
-		goto out_err;
+		goto out;
 	}
 
 	if (((custom = action_http_after_res_custom(args[0])) != NULL)) {
@@ -295,7 +312,8 @@ struct act_rule *parse_http_after_res_cond(const char **args, const char *file,
 
 	return rule;
  out_err:
-	free(rule);
+	free_act_rule(rule);
+ out:
 	return NULL;
 }
 
@@ -472,11 +490,6 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 	return NULL;
 }
 
-__attribute__((constructor))
-static void __http_rules_init(void)
-{
-}
-
 /*
  * Local variables:
  *  c-indent-level: 8
