@@ -14,12 +14,13 @@
 #include <haproxy/buf.h>
 #include <haproxy/connection.h>
 #include <haproxy/pipe-t.h>
+#include <haproxy/stconn.h>
 #include <haproxy/stream.h>
 #include <haproxy/task.h>
 #include <haproxy/trace.h>
 
 struct mux_pt_ctx {
-	struct conn_stream *cs;
+	struct sedesc *sd;
 	struct connection *conn;
 	struct wait_event wait_event;
 };
@@ -90,6 +91,12 @@ static struct trace_source trace_pt __read_mostly = {
 #define TRACE_SOURCE &trace_pt
 INITCALL1(STG_REGISTER, trace_register_source, TRACE_SOURCE);
 
+/* returns the stconn associated to the stream */
+static forceinline struct stconn *pt_sc(const struct mux_pt_ctx *pt)
+{
+	return pt->sd->sc;
+}
+
 static inline void pt_trace_buf(const struct buffer *buf, size_t ofs, size_t len)
 {
 	size_t block1, block2;
@@ -123,7 +130,7 @@ static inline void pt_trace_buf(const struct buffer *buf, size_t ofs, size_t len
 
 /* the PT traces always expect that arg1, if non-null, is of type connection
  * (from which we can derive the pt context), that arg2, if non-null, is a
- * conn-stream, and that arg3, if non-null, is a buffer.
+ * stream connector, and that arg3, if non-null, is a buffer.
  */
 static void pt_trace(enum trace_level level, uint64_t mask, const struct trace_source *src,
                      const struct ist where, const struct ist func,
@@ -131,11 +138,11 @@ static void pt_trace(enum trace_level level, uint64_t mask, const struct trace_s
 {
 	const struct connection *conn = a1;
 	const struct mux_pt_ctx *ctx = conn ? conn->ctx : NULL;
-	const struct conn_stream *cs = a2;
+	const struct stconn *sc = a2;
 	const struct buffer *buf = a3;
 	const size_t *val = a4;
 
-	if (!ctx|| src->verbosity < PT_VERB_CLEAN)
+	if (!ctx || src->verbosity < PT_VERB_CLEAN)
 		return;
 
 	/* Display frontend/backend info by default */
@@ -144,14 +151,18 @@ static void pt_trace(enum trace_level level, uint64_t mask, const struct trace_s
 	if (src->verbosity == PT_VERB_CLEAN)
 		return;
 
+	if (!sc)
+		sc = pt_sc(ctx);
+
 	/* Display the value to the 4th argument (level > STATE) */
 	if (src->level > TRACE_LEVEL_STATE && val)
 		chunk_appendf(&trace_buf, " - VAL=%lu", (long)*val);
 
-	/* Display conn and cs info, if defined (pointer + flags) */
+	/* Display conn and sc info, if defined (pointer + flags) */
 	chunk_appendf(&trace_buf, " - conn=%p(0x%08x)", conn, conn->flags);
-	if (cs)
-		chunk_appendf(&trace_buf, " cs=%p(0x%08x)", cs, cs->flags);
+	chunk_appendf(&trace_buf, " sd=%p(0x%08x)", ctx->sd, se_fl_get(ctx->sd));
+	if (sc)
+		chunk_appendf(&trace_buf, " sc=%p(0x%08x)", sc, sc->flags);
 
 	if (src->verbosity == PT_VERB_MINIMAL)
 		return;
@@ -194,20 +205,18 @@ static void mux_pt_destroy(struct mux_pt_ctx *ctx)
 
 	TRACE_POINT(PT_EV_CONN_END);
 
-	if (ctx) {
-		/* The connection must be attached to this mux to be released */
-		if (ctx->conn && ctx->conn->ctx == ctx)
-			conn = ctx->conn;
+	/* The connection must be attached to this mux to be released */
+	if (ctx->conn && ctx->conn->ctx == ctx)
+		conn = ctx->conn;
 
-		TRACE_DEVEL("freeing pt context", PT_EV_CONN_END, conn);
+	tasklet_free(ctx->wait_event.tasklet);
 
-		tasklet_free(ctx->wait_event.tasklet);
-
-		if (conn && ctx->wait_event.events != 0)
-			conn->xprt->unsubscribe(conn, conn->xprt_ctx, ctx->wait_event.events,
-						&ctx->wait_event);
-		pool_free(pool_head_pt_ctx, ctx);
-	}
+	if (conn && ctx->wait_event.events != 0)
+		conn->xprt->unsubscribe(conn, conn->xprt_ctx, ctx->wait_event.events,
+					&ctx->wait_event);
+	BUG_ON(ctx->sd && !se_fl_test(ctx->sd, SE_FL_ORPHAN));
+	sedesc_free(ctx->sd);
+	pool_free(pool_head_pt_ctx, ctx);
 
 	if (conn) {
 		conn->mux = NULL;
@@ -229,8 +238,8 @@ struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned int status)
 {
 	struct mux_pt_ctx *ctx = tctx;
 
-	TRACE_ENTER(PT_EV_CONN_WAKE, ctx->conn, ctx->cs);
-	if (ctx->cs) {
+	TRACE_ENTER(PT_EV_CONN_WAKE, ctx->conn);
+	if (!se_fl_test(ctx->sd, SE_FL_ORPHAN)) {
 		/* There's a small race condition.
 		 * mux_pt_io_cb() is only supposed to be called if we have no
 		 * stream attached. However, maybe the tasklet got woken up,
@@ -243,9 +252,9 @@ struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned int status)
 			ctx->conn->subs->events = 0;
 			tasklet_wakeup(ctx->conn->subs->tasklet);
 			ctx->conn->subs = NULL;
-		} else if (ctx->cs->data_cb->wake)
-			ctx->cs->data_cb->wake(ctx->cs);
-		TRACE_DEVEL("leaving waking up CS", PT_EV_CONN_WAKE, ctx->conn, ctx->cs);
+		} else if (pt_sc(ctx)->app_ops->wake)
+			pt_sc(ctx)->app_ops->wake(pt_sc(ctx));
+		TRACE_DEVEL("leaving waking up SC", PT_EV_CONN_WAKE, ctx->conn);
 		return t;
 	}
 	conn_ctrl_drain(ctx->conn);
@@ -263,15 +272,15 @@ struct task *mux_pt_io_cb(struct task *t, void *tctx, unsigned int status)
 	return t;
 }
 
-/* Initialize the mux once it's attached. It is expected that conn->ctx
- * points to the existing conn_stream (for outgoing connections) or NULL (for
+/* Initialize the mux once it's attached. It is expected that conn->ctx points
+ * to the existing stream connector (for outgoing connections) or NULL (for
  * incoming ones, in which case one will be allocated and a new stream will be
  * instantiated). Returns < 0 on error.
  */
 static int mux_pt_init(struct connection *conn, struct proxy *prx, struct session *sess,
 		       struct buffer *input)
 {
-	struct conn_stream *cs = conn->ctx;
+	struct stconn *sc = conn->ctx;
 	struct mux_pt_ctx *ctx = pool_alloc(pool_head_pt_ctx);
 
 	TRACE_ENTER(PT_EV_CONN_NEW);
@@ -289,31 +298,39 @@ static int mux_pt_init(struct connection *conn, struct proxy *prx, struct sessio
 	ctx->wait_event.events = 0;
 	ctx->conn = conn;
 
-	if (!cs) {
-		cs = cs_new(conn, conn->target);
-		if (!cs) {
-			TRACE_ERROR("CS allocation failure", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn);
+	if (!sc) {
+		ctx->sd = sedesc_new();
+		if (!ctx->sd) {
+			TRACE_ERROR("SC allocation failure", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn);
 			goto fail_free_ctx;
 		}
+		ctx->sd->se     = ctx;
+		ctx->sd->conn   = conn;
+		se_fl_set(ctx->sd, SE_FL_T_MUX | SE_FL_ORPHAN);
 
-		if (stream_create_from_cs(cs, &BUF_NULL) < 0) {
-			TRACE_ERROR("stream creation failure", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn, cs);
-			goto fail_free;
+		sc = sc_new_from_endp(ctx->sd, sess, input);
+		if (!sc) {
+			TRACE_ERROR("SC allocation failure", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn);
+			goto fail_free_sd;
 		}
-		TRACE_POINT(PT_EV_STRM_NEW, conn, cs);
+		TRACE_POINT(PT_EV_STRM_NEW, conn, sc);
+	}
+	else {
+		if (sc_attach_mux(sc, ctx, conn) < 0)
+			goto fail_free_ctx;
+		ctx->sd = sc->sedesc;
 	}
 	conn->ctx = ctx;
-	ctx->cs = cs;
-	cs->flags |= CS_FL_RCV_MORE;
+	se_fl_set(ctx->sd, SE_FL_RCV_MORE);
 	if (global.tune.options & GTUNE_USE_SPLICE)
-		cs->flags |= CS_FL_MAY_SPLICE;
+		se_fl_set(ctx->sd, SE_FL_MAY_SPLICE);
 
-	TRACE_LEAVE(PT_EV_CONN_NEW, conn, cs);
+	TRACE_LEAVE(PT_EV_CONN_NEW, conn);
 	return 0;
 
- fail_free:
-	cs_free(cs);
-fail_free_ctx:
+ fail_free_sd:
+	sedesc_free(ctx->sd);
+ fail_free_ctx:
 	if (ctx->wait_event.tasklet)
 		tasklet_free(ctx->wait_event.tasklet);
 	pool_free(pool_head_pt_ctx, ctx);
@@ -328,15 +345,14 @@ fail_free_ctx:
 static int mux_pt_wake(struct connection *conn)
 {
 	struct mux_pt_ctx *ctx = conn->ctx;
-	struct conn_stream *cs = ctx->cs;
 	int ret = 0;
 
-	TRACE_ENTER(PT_EV_CONN_WAKE, ctx->conn, cs);
-	if (cs) {
-		ret = cs->data_cb->wake ? cs->data_cb->wake(cs) : 0;
+	TRACE_ENTER(PT_EV_CONN_WAKE, ctx->conn);
+	if (!se_fl_test(ctx->sd, SE_FL_ORPHAN)) {
+		ret = pt_sc(ctx)->app_ops->wake ? pt_sc(ctx)->app_ops->wake(pt_sc(ctx)) : 0;
 
 		if (ret < 0) {
-			TRACE_DEVEL("leaving waking up CS", PT_EV_CONN_WAKE, ctx->conn, cs);
+			TRACE_DEVEL("leaving waking up SC", PT_EV_CONN_WAKE, ctx->conn);
 			return ret;
 		}
 	} else {
@@ -363,39 +379,30 @@ static int mux_pt_wake(struct connection *conn)
  * Attach a new stream to a connection
  * (Used for outgoing connections)
  */
-static struct conn_stream *mux_pt_attach(struct connection *conn, struct session *sess)
+static int mux_pt_attach(struct connection *conn, struct sedesc *sd, struct session *sess)
 {
-	struct conn_stream *cs;
 	struct mux_pt_ctx *ctx = conn->ctx;
 
 	TRACE_ENTER(PT_EV_STRM_NEW, conn);
 	if (ctx->wait_event.events)
 		conn->xprt->unsubscribe(ctx->conn, conn->xprt_ctx, SUB_RETRY_RECV, &ctx->wait_event);
-	cs = cs_new(conn, conn->target);
-	if (!cs) {
-		TRACE_ERROR("CS allocation failure", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn);
-		goto fail;
-	}
+	if (sc_attach_mux(sd->sc, ctx, conn) < 0)
+		return -1;
+	ctx->sd = sd;
+	se_fl_set(ctx->sd, SE_FL_RCV_MORE);
 
-	ctx->cs = cs;
-	cs->flags |= CS_FL_RCV_MORE;
-
-	TRACE_LEAVE(PT_EV_STRM_NEW, conn, cs);
-	return (cs);
-fail:
-	TRACE_DEVEL("leaving on error", PT_EV_STRM_NEW|PT_EV_STRM_END|PT_EV_STRM_ERR, conn);
-	return NULL;
+	TRACE_LEAVE(PT_EV_STRM_NEW, conn, sd->sc);
+	return 0;
 }
 
-/* Retrieves a valid conn_stream from this connection, or returns NULL. For
- * this mux, it's easy as we can only store a single conn_stream.
+/* Retrieves a valid stream connector from this connection, or returns NULL.
+ * For this mux, it's easy as we can only store a single stream connector.
  */
-static const struct conn_stream *mux_pt_get_first_cs(const struct connection *conn)
+static struct stconn *mux_pt_get_first_sc(const struct connection *conn)
 {
 	struct mux_pt_ctx *ctx = conn->ctx;
-	struct conn_stream *cs = ctx->cs;
 
-	return cs;
+	return pt_sc(ctx);
 }
 
 /* Destroy the mux and the associated connection if still attached to this mux
@@ -404,29 +411,34 @@ static void mux_pt_destroy_meth(void *ctx)
 {
 	struct mux_pt_ctx *pt = ctx;
 
-	TRACE_POINT(PT_EV_CONN_END, pt->conn, pt->cs);
-	if (!(pt->cs) || !(pt->conn) || pt->conn->ctx != pt)
+	TRACE_POINT(PT_EV_CONN_END, pt->conn, pt_sc(pt));
+	if (se_fl_test(pt->sd, SE_FL_ORPHAN) || pt->conn->ctx != pt) {
+		if (pt->conn->ctx != pt) {
+			pt->sd = NULL;
+		}
 		mux_pt_destroy(pt);
+	}
 }
 
 /*
  * Detach the stream from the connection and possibly release the connection.
  */
-static void mux_pt_detach(struct conn_stream *cs)
+static void mux_pt_detach(struct sedesc *sd)
 {
-	struct connection *conn = cs->conn;
-	struct mux_pt_ctx *ctx = cs->conn->ctx;
+	struct connection *conn = sd->conn;
+	struct mux_pt_ctx *ctx;
 
-	TRACE_ENTER(PT_EV_STRM_END, conn, cs);
+	TRACE_ENTER(PT_EV_STRM_END, conn, sd->sc);
+
+	ctx = conn->ctx;
 
 	/* Subscribe, to know if we got disconnected */
 	if (!conn_is_back(conn) && conn->owner != NULL &&
 	    !(conn->flags & (CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_SOCK_WR_SH))) {
-		ctx->cs = NULL;
 		conn->xprt->subscribe(conn, conn->xprt_ctx, SUB_RETRY_RECV, &ctx->wait_event);
 	} else {
 		/* There's no session attached to that connection, destroy it */
-		TRACE_DEVEL("killing dead connection", PT_EV_STRM_END, conn, cs);
+		TRACE_DEVEL("killing dead connection", PT_EV_STRM_END, conn, sd->sc);
 		mux_pt_destroy(ctx);
 	}
 
@@ -438,7 +450,7 @@ static int mux_pt_used_streams(struct connection *conn)
 {
 	struct mux_pt_ctx *ctx = conn->ctx;
 
-	return ctx->cs ? 1 : 0;
+	return (!se_fl_test(ctx->sd, SE_FL_ORPHAN) ? 1 : 0);
 }
 
 /* returns the number of streams still available on a connection */
@@ -447,39 +459,45 @@ static int mux_pt_avail_streams(struct connection *conn)
 	return 1 - mux_pt_used_streams(conn);
 }
 
-static void mux_pt_shutr(struct conn_stream *cs, enum cs_shr_mode mode)
+static void mux_pt_shutr(struct stconn *sc, enum co_shr_mode mode)
 {
-	TRACE_ENTER(PT_EV_STRM_SHUT, cs->conn, cs);
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 
-	if (cs->flags & CS_FL_SHR)
+	TRACE_ENTER(PT_EV_STRM_SHUT, conn, sc);
+
+	if (se_fl_test(ctx->sd, SE_FL_SHR))
 		return;
-	cs->flags &= ~(CS_FL_RCV_MORE | CS_FL_WANT_ROOM);
-	if (conn_xprt_ready(cs->conn) && cs->conn->xprt->shutr)
-		cs->conn->xprt->shutr(cs->conn, cs->conn->xprt_ctx,
-		    (mode == CS_SHR_DRAIN));
-	else if (mode == CS_SHR_DRAIN)
-		conn_ctrl_drain(cs->conn);
-	if (cs->flags & CS_FL_SHW)
-		conn_full_close(cs->conn);
+	se_fl_clr(ctx->sd, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
+	if (conn_xprt_ready(conn) && conn->xprt->shutr)
+		conn->xprt->shutr(conn, conn->xprt_ctx,
+		    (mode == CO_SHR_DRAIN));
+	else if (mode == CO_SHR_DRAIN)
+		conn_ctrl_drain(conn);
+	if (se_fl_test(ctx->sd, SE_FL_SHW))
+		conn_full_close(conn);
 
-	TRACE_LEAVE(PT_EV_STRM_SHUT, cs->conn, cs);
+	TRACE_LEAVE(PT_EV_STRM_SHUT, conn, sc);
 }
 
-static void mux_pt_shutw(struct conn_stream *cs, enum cs_shw_mode mode)
+static void mux_pt_shutw(struct stconn *sc, enum co_shw_mode mode)
 {
-	TRACE_ENTER(PT_EV_STRM_SHUT, cs->conn, cs);
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 
-	if (cs->flags & CS_FL_SHW)
+	TRACE_ENTER(PT_EV_STRM_SHUT, conn, sc);
+
+	if (se_fl_test(ctx->sd, SE_FL_SHW))
 		return;
-	if (conn_xprt_ready(cs->conn) && cs->conn->xprt->shutw)
-		cs->conn->xprt->shutw(cs->conn, cs->conn->xprt_ctx,
-		    (mode == CS_SHW_NORMAL));
-	if (!(cs->flags & CS_FL_SHR))
-		conn_sock_shutw(cs->conn, (mode == CS_SHW_NORMAL));
+	if (conn_xprt_ready(conn) && conn->xprt->shutw)
+		conn->xprt->shutw(conn, conn->xprt_ctx,
+		    (mode == CO_SHW_NORMAL));
+	if (!se_fl_test(ctx->sd, SE_FL_SHR))
+		conn_sock_shutw(conn, (mode == CO_SHW_NORMAL));
 	else
-		conn_full_close(cs->conn);
+		conn_full_close(conn);
 
-	TRACE_LEAVE(PT_EV_STRM_SHUT, cs->conn, cs);
+	TRACE_LEAVE(PT_EV_STRM_SHUT, conn, sc);
 }
 
 /*
@@ -495,51 +513,55 @@ static void mux_pt_shutw(struct conn_stream *cs, enum cs_shw_mode mode)
  * mux it may optimize the data copy to <buf> if necessary. Otherwise, it should
  * copy as much data as possible.
  */
-static size_t mux_pt_rcv_buf(struct conn_stream *cs, struct buffer *buf, size_t count, int flags)
+static size_t mux_pt_rcv_buf(struct stconn *sc, struct buffer *buf, size_t count, int flags)
 {
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 	size_t ret = 0;
 
-	TRACE_ENTER(PT_EV_RX_DATA, cs->conn, cs, buf, (size_t[]){count});
+	TRACE_ENTER(PT_EV_RX_DATA, conn, sc, buf, (size_t[]){count});
 
 	if (!count) {
-		cs->flags |= (CS_FL_RCV_MORE | CS_FL_WANT_ROOM);
+		se_fl_set(ctx->sd, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
 		goto end;
 	}
 	b_realign_if_empty(buf);
-	ret = cs->conn->xprt->rcv_buf(cs->conn, cs->conn->xprt_ctx, buf, count, flags);
-	if (conn_xprt_read0_pending(cs->conn)) {
-		cs->flags &= ~(CS_FL_RCV_MORE | CS_FL_WANT_ROOM);
-		cs->flags |= CS_FL_EOS;
-		TRACE_DEVEL("read0 on connection", PT_EV_RX_DATA, cs->conn, cs);
+	ret = conn->xprt->rcv_buf(conn, conn->xprt_ctx, buf, count, flags);
+	if (conn_xprt_read0_pending(conn)) {
+		se_fl_clr(ctx->sd, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
+		se_fl_set(ctx->sd, SE_FL_EOS);
+		TRACE_DEVEL("read0 on connection", PT_EV_RX_DATA, conn, sc);
 	}
-	if (cs->conn->flags & CO_FL_ERROR) {
-		cs->flags &= ~(CS_FL_RCV_MORE | CS_FL_WANT_ROOM);
-		cs->flags |= CS_FL_ERROR;
-		TRACE_DEVEL("error on connection", PT_EV_RX_DATA|PT_EV_CONN_ERR, cs->conn, cs);
+	if (conn->flags & CO_FL_ERROR) {
+		se_fl_clr(ctx->sd, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
+		se_fl_set(ctx->sd, SE_FL_ERROR);
+		TRACE_DEVEL("error on connection", PT_EV_RX_DATA|PT_EV_CONN_ERR, conn, sc);
 	}
   end:
-	TRACE_LEAVE(PT_EV_RX_DATA, cs->conn, cs, buf, (size_t[]){ret});
+	TRACE_LEAVE(PT_EV_RX_DATA, conn, sc, buf, (size_t[]){ret});
 	return ret;
 }
 
 /* Called from the upper layer, to send data */
-static size_t mux_pt_snd_buf(struct conn_stream *cs, struct buffer *buf, size_t count, int flags)
+static size_t mux_pt_snd_buf(struct stconn *sc, struct buffer *buf, size_t count, int flags)
 {
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 	size_t ret;
 
-	TRACE_ENTER(PT_EV_TX_DATA, cs->conn, cs, buf, (size_t[]){count});
+	TRACE_ENTER(PT_EV_TX_DATA, conn, sc, buf, (size_t[]){count});
 
-	ret = cs->conn->xprt->snd_buf(cs->conn, cs->conn->xprt_ctx, buf, count, flags);
+	ret = conn->xprt->snd_buf(conn, conn->xprt_ctx, buf, count, flags);
 
 	if (ret > 0)
 		b_del(buf, ret);
 
-	if (cs->conn->flags & CO_FL_ERROR) {
-		cs->flags |= CS_FL_ERROR;
-		TRACE_DEVEL("error on connection", PT_EV_TX_DATA|PT_EV_CONN_ERR, cs->conn, cs);
+	if (conn->flags & CO_FL_ERROR) {
+		se_fl_set(ctx->sd, SE_FL_ERROR);
+		TRACE_DEVEL("error on connection", PT_EV_TX_DATA|PT_EV_CONN_ERR, conn, sc);
 	}
 
-	TRACE_LEAVE(PT_EV_TX_DATA, cs->conn, cs, buf, (size_t[]){ret});
+	TRACE_LEAVE(PT_EV_TX_DATA, conn, sc, buf, (size_t[]){ret});
 	return ret;
 }
 
@@ -548,58 +570,66 @@ static size_t mux_pt_snd_buf(struct conn_stream *cs, struct buffer *buf, size_t
  * as at least one event is still subscribed. The <event_type> must only be a
  * combination of SUB_RETRY_RECV and SUB_RETRY_SEND. It always returns 0.
  */
-static int mux_pt_subscribe(struct conn_stream *cs, int event_type, struct wait_event *es)
+static int mux_pt_subscribe(struct stconn *sc, int event_type, struct wait_event *es)
 {
-	TRACE_POINT(PT_EV_RX_DATA|PT_EV_TX_DATA, cs->conn, cs, 0, (size_t[]){event_type});
-	return cs->conn->xprt->subscribe(cs->conn, cs->conn->xprt_ctx, event_type, es);
+	struct connection *conn = __sc_conn(sc);
+
+	TRACE_POINT(PT_EV_RX_DATA|PT_EV_TX_DATA, conn, sc, 0, (size_t[]){event_type});
+	return conn->xprt->subscribe(conn, conn->xprt_ctx, event_type, es);
 }
 
 /* Called from the upper layer, to unsubscribe <es> from events <event_type>.
  * The <es> pointer is not allowed to differ from the one passed to the
  * subscribe() call. It always returns zero.
  */
-static int mux_pt_unsubscribe(struct conn_stream *cs, int event_type, struct wait_event *es)
+static int mux_pt_unsubscribe(struct stconn *sc, int event_type, struct wait_event *es)
 {
-	TRACE_POINT(PT_EV_RX_DATA|PT_EV_TX_DATA, cs->conn, cs, 0, (size_t[]){event_type});
-	return cs->conn->xprt->unsubscribe(cs->conn, cs->conn->xprt_ctx, event_type, es);
+	struct connection *conn = __sc_conn(sc);
+
+	TRACE_POINT(PT_EV_RX_DATA|PT_EV_TX_DATA, conn, sc, 0, (size_t[]){event_type});
+	return conn->xprt->unsubscribe(conn, conn->xprt_ctx, event_type, es);
 }
 
 #if defined(USE_LINUX_SPLICE)
 /* Send and get, using splicing */
-static int mux_pt_rcv_pipe(struct conn_stream *cs, struct pipe *pipe, unsigned int count)
+static int mux_pt_rcv_pipe(struct stconn *sc, struct pipe *pipe, unsigned int count)
 {
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 	int ret;
 
-	TRACE_ENTER(PT_EV_RX_DATA, cs->conn, cs, 0, (size_t[]){count});
+	TRACE_ENTER(PT_EV_RX_DATA, conn, sc, 0, (size_t[]){count});
 
-	ret = cs->conn->xprt->rcv_pipe(cs->conn, cs->conn->xprt_ctx, pipe, count);
-	if (conn_xprt_read0_pending(cs->conn))  {
-		cs->flags |= CS_FL_EOS;
-		TRACE_DEVEL("read0 on connection", PT_EV_RX_DATA, cs->conn, cs);
+	ret = conn->xprt->rcv_pipe(conn, conn->xprt_ctx, pipe, count);
+	if (conn_xprt_read0_pending(conn))  {
+		se_fl_set(ctx->sd, SE_FL_EOS);
+		TRACE_DEVEL("read0 on connection", PT_EV_RX_DATA, conn, sc);
 	}
-	if (cs->conn->flags & CO_FL_ERROR) {
-		cs->flags |= CS_FL_ERROR;
-		TRACE_DEVEL("error on connection", PT_EV_RX_DATA|PT_EV_CONN_ERR, cs->conn, cs);
+	if (conn->flags & CO_FL_ERROR) {
+		se_fl_set(ctx->sd, SE_FL_ERROR);
+		TRACE_DEVEL("error on connection", PT_EV_RX_DATA|PT_EV_CONN_ERR, conn, sc);
 	}
 
-	TRACE_LEAVE(PT_EV_RX_DATA, cs->conn, cs, 0, (size_t[]){ret});
+	TRACE_LEAVE(PT_EV_RX_DATA, conn, sc, 0, (size_t[]){ret});
 	return (ret);
 }
 
-static int mux_pt_snd_pipe(struct conn_stream *cs, struct pipe *pipe)
+static int mux_pt_snd_pipe(struct stconn *sc, struct pipe *pipe)
 {
+	struct connection *conn = __sc_conn(sc);
+	struct mux_pt_ctx *ctx = conn->ctx;
 	int ret;
 
-	TRACE_ENTER(PT_EV_TX_DATA, cs->conn, cs, 0, (size_t[]){pipe->data});
+	TRACE_ENTER(PT_EV_TX_DATA, conn, sc, 0, (size_t[]){pipe->data});
 
-	ret = cs->conn->xprt->snd_pipe(cs->conn, cs->conn->xprt_ctx, pipe);
+	ret = conn->xprt->snd_pipe(conn, conn->xprt_ctx, pipe);
 
-	if (cs->conn->flags & CO_FL_ERROR) {
-		cs->flags |= CS_FL_ERROR;
-		TRACE_DEVEL("error on connection", PT_EV_TX_DATA|PT_EV_CONN_ERR, cs->conn, cs);
+	if (conn->flags & CO_FL_ERROR) {
+		se_fl_set(ctx->sd, SE_FL_ERROR);
+		TRACE_DEVEL("error on connection", PT_EV_TX_DATA|PT_EV_CONN_ERR, conn, sc);
 	}
 
-	TRACE_LEAVE(PT_EV_TX_DATA, cs->conn, cs, 0, (size_t[]){ret});
+	TRACE_LEAVE(PT_EV_TX_DATA, conn, sc, 0, (size_t[]){ret});
 	return ret;
 }
 #endif
@@ -632,7 +662,7 @@ const struct mux_ops mux_tcp_ops = {
 	.snd_pipe = mux_pt_snd_pipe,
 #endif
 	.attach = mux_pt_attach,
-	.get_first_cs = mux_pt_get_first_cs,
+	.get_first_sc = mux_pt_get_first_sc,
 	.detach = mux_pt_detach,
 	.avail_streams = mux_pt_avail_streams,
 	.used_streams = mux_pt_used_streams,
@@ -657,7 +687,7 @@ const struct mux_ops mux_pt_ops = {
 	.snd_pipe = mux_pt_snd_pipe,
 #endif
 	.attach = mux_pt_attach,
-	.get_first_cs = mux_pt_get_first_cs,
+	.get_first_sc = mux_pt_get_first_sc,
 	.detach = mux_pt_detach,
 	.avail_streams = mux_pt_avail_streams,
 	.used_streams = mux_pt_used_streams,
