@@ -805,6 +805,16 @@ void qcc_reset_stream(struct qcs *qcs, int err)
 	TRACE_STATE("reset stream", QMUX_EV_QCS_END, qcc->conn, qcs);
 	qcs->flags |= QC_SF_TO_RESET;
 	qcs->err = err;
+
+	/* Remove prepared stream data from connection flow-control calcul. */
+	if (qcs->tx.offset > qcs->tx.sent_offset) {
+		const uint64_t diff = qcs->tx.offset - qcs->tx.sent_offset;
+		BUG_ON(qcc->tx.offsets - diff < qcc->tx.sent_offsets);
+		qcc->tx.offsets -= diff;
+		/* Reset qcs offset to prevent BUG_ON() on qcs_destroy(). */
+		qcs->tx.offset = qcs->tx.sent_offset;
+	}
+
 	tasklet_wakeup(qcc->wait_event.tasklet);
 }
 
@@ -1183,6 +1193,11 @@ static void qcs_destroy(struct qcs *qcs)
 
 	TRACE_ENTER(QMUX_EV_QCS_END, conn, qcs);
 
+	/* MUST not removed a stream with sending prepared data left. This is
+	 * to ensure consistency on connection flow-control calculation.
+	 */
+	BUG_ON(qcs->tx.offset < qcs->tx.sent_offset);
+
 	if (quic_stream_is_remote(qcs->qcc, id))
 		qcc_release_remote_stream(qcs->qcc, id);
 
@@ -1766,19 +1781,21 @@ static void qc_release(struct qcc *qcc)
 
 	TRACE_ENTER(QMUX_EV_QCC_END, conn);
 
-	if (qcc->app_ops && qcc->app_ops->shutdown) {
-		/* Application protocol with dedicated connection closing
-		 * procedure.
-		 */
-		qcc->app_ops->shutdown(qcc->ctx);
+	if (!(qcc->flags & QC_CF_CC_EMIT)) {
+		if (qcc->app_ops && qcc->app_ops->shutdown) {
+			/* Application protocol with dedicated connection closing
+			 * procedure.
+			 */
+			qcc->app_ops->shutdown(qcc->ctx);
 
-		/* useful if application protocol should emit some closing
-		 * frames. For example HTTP/3 GOAWAY frame.
-		 */
-		qc_send(qcc);
-	}
-	else {
-		qcc_emit_cc_app(qcc, QC_ERR_NO_ERROR, 0);
+			/* useful if application protocol should emit some closing
+			 * frames. For example HTTP/3 GOAWAY frame.
+			 */
+			qc_send(qcc);
+		}
+		else {
+			qcc_emit_cc_app(qcc, QC_ERR_NO_ERROR, 0);
+		}
 	}
 
 	if (qcc->task) {
