@@ -26,6 +26,8 @@
 #include <haproxy/global.h>
 #include <haproxy/mworker.h>
 #include <haproxy/task.h>
+#include <haproxy/time.h>
+#include <haproxy/tools.h>
 
 
 static int use_program = 0; /* do we use the program section ? */
@@ -158,7 +160,6 @@ int cfg_parse_program(const char *file, int linenum, char **args, int kwm)
 		ext_child->path = NULL;
 		ext_child->id = NULL;
 		ext_child->pid = -1;
-		ext_child->relative_pid = -1;
 		ext_child->reloads = 0;
 		ext_child->timestamp = -1;
 		ext_child->ipc_fd[0] = -1;
