@@ -24,6 +24,8 @@
 #include <netinet/udp.h>
 #include <netinet/in.h>
 
+#include <import/ebtree-t.h>
+
 #include <haproxy/api.h>
 #include <haproxy/arg.h>
 #include <haproxy/cbuf.h>
@@ -40,11 +42,12 @@
 #include <haproxy/proto_quic.h>
 #include <haproxy/proto_udp.h>
 #include <haproxy/proxy-t.h>
-#include <haproxy/sock.h>
+#include <haproxy/quic_conn.h>
 #include <haproxy/quic_sock.h>
+#include <haproxy/sock.h>
 #include <haproxy/sock_inet.h>
+#include <haproxy/task.h>
 #include <haproxy/tools.h>
-#include <haproxy/xprt_quic.h>
 
 /* per-thread quic datagram handlers */
 struct quic_dghdlr *quic_dghdlrs;
@@ -529,74 +532,20 @@ static void quic_add_listener(struct protocol *proto, struct listener *listener)
 	default_add_listener(proto, listener);
 }
 
-/* Allocate the TX ring buffers for <l> listener.
- * Return 1 if succeeded, 0 if not.
- */
-static int quic_alloc_tx_rings_listener(struct listener *l)
-{
-	struct qring *qr;
-	int i;
-
-	l->rx.tx_qrings = calloc(global.nbthread, sizeof *l->rx.tx_qrings);
-	if (!l->rx.tx_qrings)
-		return 0;
-
-	MT_LIST_INIT(&l->rx.tx_qring_list);
-	for (i = 0; i < global.nbthread; i++) {
-		unsigned char *buf;
-		struct qring *qr;
-
-		qr = calloc(1, sizeof *qr);
-		if (!qr)
-			goto err;
-
-		buf = pool_alloc(pool_head_quic_tx_ring);
-		if (!buf) {
-			free(qr);
-			goto err;
-		}
-
-		qr->cbuf = cbuf_new(buf, QUIC_TX_RING_BUFSZ);
-		if (!qr->cbuf) {
-			pool_free(pool_head_quic_tx_ring, buf);
-			free(qr);
-			goto err;
-		}
-
-        l->rx.tx_qrings[i] = qr;
-		MT_LIST_APPEND(&l->rx.tx_qring_list, &qr->mt_list);
-	}
-
-	return 1;
-
- err:
-	while ((qr = MT_LIST_POP(&l->rx.tx_qring_list, typeof(qr), mt_list))) {
-		pool_free(pool_head_quic_tx_ring, qr->cbuf->buf);
-		cbuf_free(qr->cbuf);
-		free(qr);
-	}
-	free(l->rx.tx_qrings);
-	return 0;
-}
-
 /* Allocate the RX buffers for <l> listener.
  * Return 1 if succeeded, 0 if not.
  */
 static int quic_alloc_rxbufs_listener(struct listener *l)
 {
 	int i;
-	struct rxbuf *rxbuf;
-
-	l->rx.rxbufs = calloc(global.nbthread, sizeof *l->rx.rxbufs);
-	if (!l->rx.rxbufs)
-		return 0;
+	struct quic_receiver_buf *tmp;
 
 	MT_LIST_INIT(&l->rx.rxbuf_list);
 	for (i = 0; i < global.nbthread; i++) {
+		struct quic_receiver_buf *rxbuf;
 		char *buf;
-		struct rxbuf *rxbuf;
 
-		rxbuf = calloc(1, sizeof *rxbuf);
+		rxbuf = calloc(1, sizeof(*rxbuf));
 		if (!rxbuf)
 			goto err;
 
@@ -606,21 +555,18 @@ static int quic_alloc_rxbufs_listener(struct listener *l)
 			goto err;
 		}
 
-		l->rx.rxbufs[i] = rxbuf;
-
 		rxbuf->buf = b_make(buf, QUIC_RX_BUFSZ, 0, 0);
-		LIST_INIT(&rxbuf->dgrams);
-		MT_LIST_APPEND(&l->rx.rxbuf_list, &rxbuf->mt_list);
+		LIST_INIT(&rxbuf->dgram_list);
+		MT_LIST_APPEND(&l->rx.rxbuf_list, &rxbuf->rxbuf_el);
 	}
 
 	return 1;
 
  err:
-	while ((rxbuf = MT_LIST_POP(&l->rx.rxbuf_list, typeof(rxbuf), mt_list))) {
-		pool_free(pool_head_quic_rxbuf, rxbuf->buf.area);
-		free(rxbuf);
+	while ((tmp = MT_LIST_POP(&l->rx.rxbuf_list, typeof(tmp), rxbuf_el))) {
+		pool_free(pool_head_quic_rxbuf, tmp->buf.area);
+		free(tmp);
 	}
-	free(l->rx.rxbufs);
 	return 0;
 }
 
@@ -639,7 +585,8 @@ static int quic_alloc_rxbufs_listener(struct listener *l)
  */
 static int quic_bind_listener(struct listener *listener, char *errmsg, int errlen)
 {
-	int err = ERR_NONE;
+	const struct sockaddr_storage addr = listener->rx.addr;
+	int fd, err = ERR_NONE;
 	char *msg = NULL;
 
 	/* ensure we never return garbage */
@@ -654,8 +601,26 @@ static int quic_bind_listener(struct listener *listener, char *errmsg, int errle
 		goto udp_return;
 	}
 
-	if (!quic_alloc_tx_rings_listener(listener) ||
-	    !quic_alloc_rxbufs_listener(listener)) {
+	/* Set IP_PKTINFO to retrieve destination address on recv. */
+	fd = listener->rx.fd;
+	switch (addr.ss_family) {
+	case AF_INET:
+#if defined(IP_PKTINFO)
+		setsockopt(fd, IPPROTO_IP, IP_PKTINFO, &one, sizeof(one));
+#elif defined(IP_RECVDSTADDR)
+		setsockopt(fd, IPPROTO_IP, IP_RECVDSTADDR, &one, sizeof(one));
+#endif /* IP_PKTINFO || IP_RECVDSTADDR */
+		break;
+	case AF_INET6:
+#ifdef IPV6_RECVPKTINFO
+		setsockopt(fd, IPPROTO_IPV6, IPV6_RECVPKTINFO, &one, sizeof(one));
+#endif
+		break;
+	default:
+		break;
+	}
+
+	if (!quic_alloc_rxbufs_listener(listener)) {
 		msg = "could not initialize tx/rx rings";
 		err |= ERR_WARN;
 		goto udp_return;
