@@ -1,9 +1,11 @@
 #include <import/eb64tree.h>
 
+#include <haproxy/quic_conn-t.h>
 #include <haproxy/quic_loss.h>
-#include <haproxy/xprt_quic-t.h>
+#include <haproxy/quic_tls.h>
 
 #include <haproxy/atomic.h>
+#include <haproxy/list.h>
 #include <haproxy/ticks.h>
 #include <haproxy/trace.h>
 
@@ -78,7 +80,7 @@ struct quic_pktns *quic_loss_pktns(struct quic_conn *qc)
  * as PTO value if not.
  */
 struct quic_pktns *quic_pto_pktns(struct quic_conn *qc,
-                                  int handshake_completed,
+                                  int handshake_confirmed,
                                   unsigned int *pto)
 {
 	int i;
@@ -95,7 +97,7 @@ struct quic_pktns *quic_pto_pktns(struct quic_conn *qc,
 		struct quic_enc_level *hel;
 
 		hel = &qc->els[QUIC_TLS_ENC_LEVEL_HANDSHAKE];
-		if (hel->tls_ctx.flags & QUIC_FL_TLS_SECRETS_SET) {
+		if (quic_tls_has_tx_sec(hel)) {
 			pktns = &qc->pktns[QUIC_TLS_PKTNS_HANDSHAKE];
 		}
 		else {
@@ -115,7 +117,7 @@ struct quic_pktns *quic_pto_pktns(struct quic_conn *qc,
 			continue;
 
 		if (i == QUIC_TLS_PKTNS_01RTT) {
-			if (!handshake_completed) {
+			if (!handshake_confirmed) {
 				TRACE_STATE("handshake not already completed", QUIC_EV_CONN_SPTO, qc);
 				pktns = p;
 				goto out;
