@@ -29,9 +29,6 @@
 #include <haproxy/freq_ctr.h>
 #include <haproxy/global.h>
 #include <haproxy/pipe.h>
-#include <haproxy/stream_interface.h>
-#include <haproxy/ticks.h>
-#include <haproxy/time.h>
 #include <haproxy/tools.h>
 
 
@@ -61,6 +58,8 @@ int raw_sock_to_pipe(struct connection *conn, void *xprt_ctx, struct pipe *pipe,
 	if (!conn_ctrl_ready(conn))
 		return 0;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (!fd_recv_ready(conn->handle.fd))
 		return 0;
 
@@ -95,7 +94,7 @@ int raw_sock_to_pipe(struct connection *conn, void *xprt_ctx, struct pipe *pipe,
 			if (ret == 0)
 				goto out_read0;
 
-			if (errno == EAGAIN) {
+			if (errno == EAGAIN || errno == EWOULDBLOCK) {
 				/* there are two reasons for EAGAIN :
 				 *   - nothing in the socket buffer (standard)
 				 *   - pipe is full
@@ -174,6 +173,8 @@ int raw_sock_from_pipe(struct connection *conn, void *xprt_ctx, struct pipe *pip
 	if (!conn_ctrl_ready(conn))
 		return 0;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (!fd_send_ready(conn->handle.fd))
 		return 0;
 
@@ -190,7 +191,7 @@ int raw_sock_from_pipe(struct connection *conn, void *xprt_ctx, struct pipe *pip
 			     SPLICE_F_MOVE|SPLICE_F_NONBLOCK);
 
 		if (ret <= 0) {
-			if (ret == 0 || errno == EAGAIN) {
+			if (ret == 0 || errno == EAGAIN || errno == EWOULDBLOCK) {
 				fd_cant_send(conn->handle.fd);
 				break;
 			}
@@ -233,6 +234,8 @@ static size_t raw_sock_to_buf(struct connection *conn, void *xprt_ctx, struct bu
 	if (!conn_ctrl_ready(conn))
 		return 0;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (!fd_recv_ready(conn->handle.fd))
 		return 0;
 
@@ -298,7 +301,7 @@ static size_t raw_sock_to_buf(struct connection *conn, void *xprt_ctx, struct bu
 		else if (ret == 0) {
 			goto read0;
 		}
-		else if (errno == EAGAIN || errno == ENOTCONN) {
+		else if (errno == EAGAIN || errno == EWOULDBLOCK || errno == ENOTCONN) {
 			/* socket buffer exhausted */
 			fd_cant_recv(conn->handle.fd);
 			break;
@@ -326,7 +329,7 @@ static size_t raw_sock_to_buf(struct connection *conn, void *xprt_ctx, struct bu
 	 * of recv()'s return value 0, so we have no way to tell there was
 	 * an error without checking.
 	 */
-	if (unlikely(fdtab[conn->handle.fd].state & FD_POLL_ERR))
+	if (unlikely(!done && fdtab[conn->handle.fd].state & FD_POLL_ERR))
 		conn->flags |= CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_SOCK_WR_SH;
 	goto leave;
 }
@@ -352,6 +355,8 @@ static size_t raw_sock_from_buf(struct connection *conn, void *xprt_ctx, const s
 	if (!conn_ctrl_ready(conn))
 		return 0;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (!fd_send_ready(conn->handle.fd))
 		return 0;
 
@@ -390,7 +395,7 @@ static size_t raw_sock_from_buf(struct connection *conn, void *xprt_ctx, const s
 			if (!count)
 				fd_stop_send(conn->handle.fd);
 		}
-		else if (ret == 0 || errno == EAGAIN || errno == ENOTCONN || errno == EINPROGRESS) {
+		else if (ret == 0 || errno == EAGAIN || errno == EWOULDBLOCK || errno == ENOTCONN || errno == EINPROGRESS) {
 			/* nothing written, we need to poll for write first */
 			fd_cant_send(conn->handle.fd);
 			break;
@@ -469,12 +474,13 @@ static struct xprt_ops raw_sock = {
 };
 
 
-__attribute__((constructor))
 static void __raw_sock_init(void)
 {
 	xprt_register(XPRT_RAW, &raw_sock);
 }
 
+INITCALL0(STG_REGISTER, __raw_sock_init);
+
 /*
  * Local variables:
  *  c-indent-level: 8
