@@ -24,9 +24,17 @@
 #include <haproxy/buf.h>
 #include <haproxy/cli.h>
 #include <haproxy/ring.h>
-#include <haproxy/stream_interface.h>
+#include <haproxy/sc_strm.h>
+#include <haproxy/stconn.h>
 #include <haproxy/thread.h>
 
+/* context used to dump the contents of a ring via "show events" or "show errors" */
+struct show_ring_ctx {
+	struct ring *ring; /* ring to be dumped */
+	size_t ofs;        /* offset to restart from, ~0 = end */
+	uint flags;        /* set of RING_WF_* */
+};
+
 /* Initialize a pre-allocated ring with the buffer area
  * of size */
 void ring_init(struct ring *ring, void *area, size_t size)
@@ -67,6 +75,30 @@ struct ring *ring_new(size_t size)
 	return NULL;
 }
 
+/* Creates a unified ring + storage area at address <area> for <size> bytes.
+ * If <area> is null, then it's allocated of the requested size. The ring
+ * struct is part of the area so the usable area is slightly reduced. However
+ * the ring storage is immediately adjacent to the struct. ring_free() will
+ * ignore such rings, so the caller is responsible for releasing them.
+ */
+struct ring *ring_make_from_area(void *area, size_t size)
+{
+	struct ring *ring = NULL;
+
+	if (size < sizeof(ring))
+		return NULL;
+
+	if (!area)
+		area = malloc(size);
+	if (!area)
+		return NULL;
+
+	ring = area;
+	area += sizeof(*ring);
+	ring_init(ring, area, size - sizeof(*ring));
+	return ring;
+}
+
 /* Resizes existing ring <ring> to <size> which must be larger, without losing
  * its contents. The new size must be at least as large as the previous one or
  * no change will be performed. The pointer to the ring is returned on success,
@@ -104,6 +136,11 @@ void ring_free(struct ring *ring)
 {
 	if (!ring)
 		return;
+
+	/* make sure it was not allocated by ring_make_from_area */
+	if (ring->buf.area == (void *)ring + sizeof(*ring))
+		return;
+
 	free(ring->buf.area);
 	free(ring);
 }
@@ -246,10 +283,12 @@ void ring_detach_appctx(struct ring *ring, struct appctx *appctx, size_t ofs)
  * meant to be used when registering a CLI function to dump a buffer, so it
  * returns zero on success, or non-zero on failure with a message in the appctx
  * CLI context. It automatically sets the io_handler and io_release callbacks if
- * they were not set.
+ * they were not set. The <flags> take a combination of RING_WF_*.
  */
-int ring_attach_cli(struct ring *ring, struct appctx *appctx)
+int ring_attach_cli(struct ring *ring, struct appctx *appctx, uint flags)
 {
+	struct show_ring_ctx *ctx = applet_reserve_svcctx(appctx, sizeof(*ctx));
+
 	if (!ring_attach(ring))
 		return cli_err(appctx,
 		               "Sorry, too many watchers (255) on this ring buffer. "
@@ -259,8 +298,11 @@ int ring_attach_cli(struct ring *ring, struct appctx *appctx)
 		appctx->io_handler = cli_io_handler_show_ring;
 	if (!appctx->io_release)
                 appctx->io_release = cli_io_release_show_ring;
-	appctx->ctx.cli.p0 = ring;
-	appctx->ctx.cli.o0 = ~0; // start from the oldest event
+
+	memset(ctx, 0, sizeof(*ctx));
+	ctx->ring  = ring;
+	ctx->ofs   = ~0; // start from the oldest event
+	ctx->flags = flags;
 	return 0;
 }
 
@@ -275,16 +317,17 @@ int ring_attach_cli(struct ring *ring, struct appctx *appctx)
  */
 int cli_io_handler_show_ring(struct appctx *appctx)
 {
-	struct stream_interface *si = appctx->owner;
-	struct ring *ring = appctx->ctx.cli.p0;
+	struct show_ring_ctx *ctx = appctx->svcctx;
+	struct stconn *sc = appctx_sc(appctx);
+	struct ring *ring = ctx->ring;
 	struct buffer *buf = &ring->buf;
-	size_t ofs = appctx->ctx.cli.o0;
+	size_t ofs = ctx->ofs;
 	size_t last_ofs;
 	uint64_t msg_len;
 	size_t len, cnt;
 	int ret;
 
-	if (unlikely(si_ic(si)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
+	if (unlikely(sc_ic(sc)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
 		return 1;
 
 	HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
@@ -305,7 +348,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 		ofs = 0;
 
 		/* going to the end means looking at tail-1 */
-		if (appctx->ctx.cli.i0 & 2)
+		if (ctx->flags & RING_WF_SEEK_NEW)
 			ofs += b_data(buf) - 1;
 
 		HA_ATOMIC_INC(b_peek(buf, ofs));
@@ -343,8 +386,7 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 		trash.data += len;
 		trash.area[trash.data++] = '\n';
 
-		if (ci_putchk(si_ic(si), &trash) == -1) {
-			si_rx_room_blk(si);
+		if (applet_putchk(appctx, &trash) == -1) {
 			ret = 0;
 			break;
 		}
@@ -354,14 +396,14 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 	HA_ATOMIC_INC(b_peek(buf, ofs));
 	ofs += ring->ofs;
 	last_ofs = ring->ofs;
-	appctx->ctx.cli.o0 = ofs;
+	ctx->ofs = ofs;
 	HA_RWLOCK_RDUNLOCK(LOGSRV_LOCK, &ring->lock);
 
-	if (ret && (appctx->ctx.cli.i0 & 1)) {
+	if (ret && (ctx->flags & RING_WF_WAIT_MODE)) {
 		/* we've drained everything and are configured to wait for more
 		 * data or an event (keypress, close)
 		 */
-		if (!si_oc(si)->output && !(si_oc(si)->flags & CF_SHUTW)) {
+		if (!sc_oc(sc)->output && !(sc_oc(sc)->flags & CF_SHUTW)) {
 			/* let's be woken up once new data arrive */
 			HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
 			LIST_APPEND(&ring->waiters, &appctx->wait_entry);
@@ -372,13 +414,13 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 				 * unlock and the lock, and the writer might not
 				 * have seen us. We need to reschedule a read.
 				 */
-				si_rx_endp_more(si);
+				applet_have_more_data(appctx);
 			} else
-				si_rx_endp_done(si);
+				applet_have_no_more_data(appctx);
 			ret = 0;
 		}
 		/* always drain all the request */
-		co_skip(si_oc(si), si_oc(si)->output);
+		co_skip(sc_oc(sc), sc_oc(sc)->output);
 	}
 	return ret;
 }
@@ -386,8 +428,9 @@ int cli_io_handler_show_ring(struct appctx *appctx)
 /* must be called after cli_io_handler_show_ring() above */
 void cli_io_release_show_ring(struct appctx *appctx)
 {
-	struct ring *ring = appctx->ctx.cli.p0;
-	size_t ofs = appctx->ctx.cli.o0;
+	struct show_ring_ctx *ctx = appctx->svcctx;
+	struct ring *ring = ctx->ring;
+	size_t ofs = ctx->ofs;
 
 	ring_detach_appctx(ring, appctx, ofs);
 }
