@@ -13,7 +13,6 @@
 #define _GNU_SOURCE
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,12 +30,29 @@
 #include <haproxy/listener.h>
 #include <haproxy/log.h>
 #include <haproxy/namespace.h>
+#include <haproxy/proto_sockpair.h>
 #include <haproxy/sock.h>
 #include <haproxy/sock_inet.h>
 #include <haproxy/tools.h>
 
+#define SOCK_XFER_OPT_FOREIGN 0x000000001
+#define SOCK_XFER_OPT_V6ONLY  0x000000002
+#define SOCK_XFER_OPT_DGRAM   0x000000004
+
 /* the list of remaining sockets transferred from an older process */
-struct xfer_sock_list *xfer_sock_list = NULL;
+struct xfer_sock_list {
+	int fd;
+	int options; /* socket options as SOCK_XFER_OPT_* */
+	char *iface;
+	char *namespace;
+	int if_namelen;
+	int ns_namelen;
+	struct xfer_sock_list *prev;
+	struct xfer_sock_list *next;
+	struct sockaddr_storage addr;
+};
+
+static struct xfer_sock_list *xfer_sock_list;
 
 
 /* Accept an incoming connection from listener <l>, and return it, as well as
@@ -79,13 +95,20 @@ struct connection *sock_accept_conn(struct listener *l, int *status)
 	{
 		laddr = sizeof(*conn->src);
 		if ((cfd = accept(l->rx.fd, (struct sockaddr*)addr, &laddr)) != -1) {
-			fcntl(cfd, F_SETFL, O_NONBLOCK);
+			fd_set_nonblock(cfd);
 			if (master)
-				fcntl(cfd, F_SETFD, FD_CLOEXEC);
+				fd_set_cloexec(cfd);
 		}
 	}
 
 	if (likely(cfd != -1)) {
+		if (unlikely(cfd >= global.maxsock)) {
+			send_log(p, LOG_EMERG,
+				 "Proxy %s reached the configured maximum connection limit. Please check the global 'maxconn' value.\n",
+				 p->id);
+			goto fail_conn;
+		}
+
 		/* Perfect, the connection was accepted */
 		conn = conn_new(&l->obj_type);
 		if (!conn)
@@ -93,7 +116,6 @@ struct connection *sock_accept_conn(struct listener *l, int *status)
 
 		conn->src = addr;
 		conn->handle.fd = cfd;
-		conn->flags |= CO_FL_ADDR_FROM_SET;
 		ret = CO_AC_DONE;
 		goto done;
 	}
@@ -102,6 +124,9 @@ struct connection *sock_accept_conn(struct listener *l, int *status)
 	sockaddr_free(&addr);
 
 	switch (errno) {
+#if defined(EWOULDBLOCK) && defined(EAGAIN) && EWOULDBLOCK != EAGAIN
+	case EWOULDBLOCK:
+#endif
 	case EAGAIN:
 		ret = CO_AC_DONE; /* nothing more to accept */
 		if (fdtab[l->rx.fd].state & (FD_POLL_HUP|FD_POLL_ERR)) {
@@ -285,6 +310,47 @@ int sock_get_old_sockets(const char *unixsocket)
 	int cur_fd = 0;
 	size_t maxoff = 0, curoff = 0;
 
+	if (strncmp("sockpair@", unixsocket, strlen("sockpair@")) == 0) {
+		/* sockpair for master-worker usage */
+		int sv[2];
+		int dst_fd;
+
+		dst_fd = strtoll(unixsocket + strlen("sockpair@"), NULL, 0);
+
+		if (socketpair(PF_UNIX, SOCK_STREAM, 0, sv) == -1) {
+			ha_warning("socketpair(): Cannot create socketpair. Giving up.\n");
+		}
+
+		if (send_fd_uxst(dst_fd, sv[0]) == -1) {
+			ha_alert("socketpair: cannot transfer socket.\n");
+			close(sv[0]);
+			close(sv[1]);
+			goto out;
+		}
+
+		close(sv[0]); /* we don't need this side anymore */
+		sock = sv[1];
+
+	} else {
+		/* Unix socket */
+
+		sock = socket(PF_UNIX, SOCK_STREAM, 0);
+		if (sock < 0) {
+			ha_warning("Failed to connect to the old process socket '%s'\n", unixsocket);
+			goto out;
+		}
+
+		strncpy(addr.sun_path, unixsocket, sizeof(addr.sun_path) - 1);
+		addr.sun_path[sizeof(addr.sun_path) - 1] = 0;
+		addr.sun_family = PF_UNIX;
+
+		ret = connect(sock, (struct sockaddr *)&addr, sizeof(addr));
+		if (ret < 0) {
+			ha_warning("Failed to connect to the old process socket '%s'\n", unixsocket);
+			goto out;
+		}
+
+	}
 	memset(&msghdr, 0, sizeof(msghdr));
 	cmsgbuf = malloc(CMSG_SPACE(sizeof(int)) * MAX_SEND_FD);
 	if (!cmsgbuf) {
@@ -292,22 +358,6 @@ int sock_get_old_sockets(const char *unixsocket)
 		goto out;
 	}
 
-	sock = socket(PF_UNIX, SOCK_STREAM, 0);
-	if (sock < 0) {
-		ha_warning("Failed to connect to the old process socket '%s'\n", unixsocket);
-		goto out;
-	}
-
-	strncpy(addr.sun_path, unixsocket, sizeof(addr.sun_path) - 1);
-	addr.sun_path[sizeof(addr.sun_path) - 1] = 0;
-	addr.sun_family = PF_UNIX;
-
-	ret = connect(sock, (struct sockaddr *)&addr, sizeof(addr));
-	if (ret < 0) {
-		ha_warning("Failed to connect to the old process socket '%s'\n", unixsocket);
-		goto out;
-	}
-
 	setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, (void *)&tv, sizeof(tv));
 	iov.iov_base = &fd_nb;
 	iov.iov_len = sizeof(fd_nb);
@@ -596,6 +646,24 @@ int sock_find_compatible_fd(const struct receiver *rx)
 	return ret;
 }
 
+/* After all protocols are bound, there may remain some old sockets that have
+ * been removed between the previous config and the new one. These ones must
+ * be dropped, otherwise they will remain open and may prevent a service from
+ * restarting.
+ */
+void sock_drop_unused_old_sockets()
+{
+	while (xfer_sock_list != NULL) {
+		struct xfer_sock_list *tmpxfer = xfer_sock_list->next;
+
+		close(xfer_sock_list->fd);
+		free(xfer_sock_list->iface);
+		free(xfer_sock_list->namespace);
+		free(xfer_sock_list);
+		xfer_sock_list = tmpxfer;
+	}
+}
+
 /* Tests if the receiver supports accepting connections. Returns positive on
  * success, 0 if not possible, negative if the socket is non-recoverable. The
  * rationale behind this is that inherited FDs may be broken and that shared
@@ -634,6 +702,7 @@ void sock_accept_iocb(int fd)
  */
 void sock_conn_ctrl_init(struct connection *conn)
 {
+	BUG_ON(conn->flags & CO_FL_FDLESS);
 	fd_insert(conn->handle.fd, conn, sock_conn_iocb, tid_bit);
 }
 
@@ -643,6 +712,7 @@ void sock_conn_ctrl_init(struct connection *conn)
  */
 void sock_conn_ctrl_close(struct connection *conn)
 {
+	BUG_ON(conn->flags & CO_FL_FDLESS);
 	fd_delete(conn->handle.fd);
 	conn->handle.fd = DEAD_FD_MAGIC;
 }
@@ -668,6 +738,8 @@ int sock_conn_check(struct connection *conn)
 	if (!(conn->flags & CO_FL_WAIT_L4_CONN))
 		return 1; /* strange we were called while ready */
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (!fd_send_ready(fd) && !(fdtab[fd].state & (FD_POLL_ERR|FD_POLL_HUP)))
 		return 0;
 
@@ -833,6 +905,8 @@ int sock_drain(struct connection *conn)
 	int fd = conn->handle.fd;
 	int len;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (fdtab[fd].state & (FD_POLL_ERR|FD_POLL_HUP))
 		goto shut;
 
@@ -852,7 +926,7 @@ int sock_drain(struct connection *conn)
 			goto shut;
 
 		if (len < 0) {
-			if (errno == EAGAIN) {
+			if (errno == EAGAIN || errno == EWOULDBLOCK) {
 				/* connection not closed yet */
 				fd_cant_recv(fd);
 				break;
@@ -885,6 +959,8 @@ int sock_check_events(struct connection *conn, int event_type)
 {
 	int ret = 0;
 
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (event_type & SUB_RETRY_RECV) {
 		if (fd_recv_ready(conn->handle.fd))
 			ret |= SUB_RETRY_RECV;
@@ -907,6 +983,8 @@ int sock_check_events(struct connection *conn, int event_type)
  */
 void sock_ignore_events(struct connection *conn, int event_type)
 {
+	BUG_ON(conn->flags & CO_FL_FDLESS);
+
 	if (event_type & SUB_RETRY_RECV)
 		fd_stop_recv(conn->handle.fd);
 
