@@ -169,8 +169,12 @@ struct stconn *sc_new_from_endp(struct sedesc *sd, struct session *sess, struct
 	if (unlikely(!sc))
 		return NULL;
 	if (unlikely(!stream_new(sess, sc, input))) {
-		pool_free(pool_head_connstream, sc);
 		sd->sc = NULL;
+		if (sc->sedesc != sd) {
+			/* none was provided so sc_new() allocated one */
+			sedesc_free(sc->sedesc);
+		}
+		pool_free(pool_head_connstream, sc);
 		se_fl_set(sd, SE_FL_ORPHAN);
 		return NULL;
 	}
@@ -254,12 +258,6 @@ int sc_attach_mux(struct stconn *sc, void *sd, void *ctx)
 	struct connection *conn = ctx;
 	struct sedesc *sedesc = sc->sedesc;
 
-	sedesc->se = sd;
-	sedesc->conn = ctx;
-	se_fl_set(sedesc, SE_FL_T_MUX);
-	se_fl_clr(sedesc, SE_FL_DETACHED);
-	if (!conn->ctx)
-		conn->ctx = sc;
 	if (sc_strm(sc)) {
 		if (!sc->wait_event.tasklet) {
 			sc->wait_event.tasklet = tasklet_new();
@@ -284,6 +282,13 @@ int sc_attach_mux(struct stconn *sc, void *sd, void *ctx)
 
 		sc->app_ops = &sc_app_check_ops;
 	}
+
+	sedesc->se = sd;
+	sedesc->conn = ctx;
+	se_fl_set(sedesc, SE_FL_T_MUX);
+	se_fl_clr(sedesc, SE_FL_DETACHED);
+	if (!conn->ctx)
+		conn->ctx = sc;
 	return 0;
 }
 
