@@ -12,7 +12,6 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,8 +34,8 @@
 #include <haproxy/proto_tcp.h>
 #include <haproxy/proxy-t.h>
 #include <haproxy/sample.h>
+#include <haproxy/sc_strm.h>
 #include <haproxy/session.h>
-#include <haproxy/stream_interface.h>
 #include <haproxy/tcp_rules.h>
 #include <haproxy/tools.h>
 
@@ -68,9 +67,9 @@ static enum act_return tcp_action_req_set_src(struct act_rule *rule, struct prox
 
 	case ACT_F_TCP_REQ_CNT:
 	case ACT_F_HTTP_REQ:
-		if (!si_get_src(&s->si[0]))
+		if (!sc_get_src(s->scf))
 			goto end;
-		src = s->si[0].src;
+		src = s->scf->src;
 		break;
 
 	default:
@@ -124,9 +123,9 @@ static enum act_return tcp_action_req_set_dst(struct act_rule *rule, struct prox
 
 	case ACT_F_TCP_REQ_CNT:
 	case ACT_F_HTTP_REQ:
-		if (!si_get_dst(&s->si[0]))
+		if (!sc_get_dst(s->scf))
 			goto end;
-		dst = s->si[0].dst;
+		dst = s->scf->dst;
 		break;
 
 	default:
@@ -181,9 +180,9 @@ static enum act_return tcp_action_req_set_src_port(struct act_rule *rule, struct
 
 	case ACT_F_TCP_REQ_CNT:
 	case ACT_F_HTTP_REQ:
-		if (!si_get_src(&s->si[0]))
+		if (!sc_get_src(s->scf))
 			goto end;
-		src = s->si[0].src;
+		src = s->scf->src;
 		break;
 
 	default:
@@ -236,9 +235,9 @@ static enum act_return tcp_action_req_set_dst_port(struct act_rule *rule, struct
 
 	case ACT_F_TCP_REQ_CNT:
 	case ACT_F_HTTP_REQ:
-		if (!si_get_dst(&s->si[0]))
+		if (!sc_get_dst(s->scf))
 			goto end;
-		dst = s->si[0].dst;
+		dst = s->scf->dst;
 		break;
 
 	default:
@@ -288,7 +287,10 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 	 * is present, returning with ERR will cause lingering to be disabled.
 	 */
 	if (strm)
-		strm->si[0].flags |= SI_FL_NOLINGER;
+		strm->scf->flags |= SC_FL_NOLINGER;
+
+	if (conn->flags & CO_FL_FDLESS)
+		goto out;
 
 	/* We're on the client-facing side, we must force to disable lingering to
 	 * ensure we will use an RST exclusively and kill any pending data.
